# Copyright (c) 2021 PaddlePaddle Authors. All Rights Reserved.
# Copyright 2021 The HuggingFace Inc. team.
#
# Licensed under the Apache License, Version 2.0 (the "License"
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import paddle
import paddle.nn as nn
import paddle.nn.functional as F
from paddle import tensor
from paddle.nn import Layer
from ..electra.modeling import get_activation
from .. import PretrainedModel, register_base_model

__all__ = [
    "ConvBertModel",
    "ConvBertPretrainedModel",
    "ConvBertForTotalPretraining",
    "ConvBertDiscriminator",
    "ConvBertGenerator",
    "ConvBertClassificationHead",
    "ConvBertForSequenceClassification",
    "ConvBertForTokenClassification",
    "ConvBertPretrainingCriterion",
    "ConvBertForQuestionAnswering",
    "ConvBertForMultipleChoice",
    "ConvBertForPretraining",
]
dtype_float = paddle.get_default_dtype()


def _convert_attention_mask(attn_mask, dtype):
    if attn_mask is not None and attn_mask.dtype != dtype:
        attn_mask_dtype = attn_mask.dtype
        if attn_mask_dtype in [paddle.bool, paddle.int8, paddle.int16, paddle.int32, paddle.int64]:
            attn_mask = (paddle.cast(attn_mask, dtype) - 1.0) * 1e9
        else:
            attn_mask = paddle.cast(attn_mask, dtype)
    return attn_mask


class GroupedLinear(nn.Layer):
    def __init__(self, input_size, output_size, num_groups):
        super().__init__()
        self.input_size = input_size
        self.output_size = output_size
        self.num_groups = num_groups
        self.group_in_dim = self.input_size // self.num_groups
        self.group_out_dim = self.output_size // self.num_groups
        self.weight = paddle.create_parameter(
            shape=[self.num_groups, self.group_in_dim, self.group_out_dim], dtype=dtype_float
        )
        self.bias = paddle.create_parameter(shape=[output_size], dtype=dtype_float, is_bias=True)

    def forward(self, hidden_states):
        batch_size = hidden_states.shape[0]
        x = tensor.reshape(hidden_states, [-1, self.num_groups, self.group_in_dim])
        x = tensor.transpose(x, perm=[1, 0, 2])
        x = tensor.matmul(x, self.weight)
        x = tensor.transpose(x, perm=[1, 0, 2])
        x = tensor.reshape(x, [batch_size, -1, self.output_size])
        x = x + self.bias
        return x


class SeparableConv1D(nn.Layer):
    """This class implements separable convolution, i.e. a depthwise and a pointwise layer"""

    def __init__(self, input_filters, output_filters, kernel_size):
        super().__init__()
        self.depthwise = nn.Conv1D(
            input_filters,
            input_filters,
            kernel_size=kernel_size,
            groups=input_filters,
            padding=kernel_size // 2,
            bias_attr=False,
            data_format="NLC",
        )
        self.pointwise = nn.Conv1D(
            input_filters,
            output_filters,
            kernel_size=1,
            bias_attr=False,
            data_format="NLC",
        )
        self.bias = paddle.create_parameter(shape=[output_filters], dtype=dtype_float, is_bias=True)

    def forward(self, hidden_states):
        x = self.depthwise(hidden_states)
        x = self.pointwise(x) + self.bias
        return x


class MultiHeadAttentionWithConv(Layer):
    def __init__(
        self,
        embed_dim,
        num_heads,
        dropout=0.0,
        kdim=None,
        vdim=None,
        need_weights=False,
        conv_kernel_size=None,
        head_ratio=None,
    ):
        super(MultiHeadAttentionWithConv, self).__init__()
        self.embed_dim = embed_dim
        self.kdim = kdim if kdim is not None else embed_dim
        self.vdim = vdim if vdim is not None else embed_dim
        self.need_weights = need_weights
        self.head_dim = embed_dim // num_heads
        self.scale = self.head_dim**-0.5
        assert self.head_dim * num_heads == self.embed_dim, "embed_dim must be divisible by num_heads"

        new_num_attention_heads = num_heads // head_ratio
        if num_heads // head_ratio < 1:
            self.num_heads = 1
            self.conv_type = "noconv"
        else:
            self.num_heads = new_num_attention_heads
            self.conv_type = "sdconv"

        self.all_head_size = self.num_heads * self.head_dim

        self.dropout = nn.Dropout(dropout)
        self.q_proj = nn.Linear(embed_dim, self.all_head_size)
        self.k_proj = nn.Linear(self.kdim, self.all_head_size)
        self.v_proj = nn.Linear(self.vdim, self.all_head_size)
        self.out_proj = nn.Linear(embed_dim, embed_dim)

        if self.conv_type == "sdconv":
            self.conv_kernel_size = conv_kernel_size
            self.key_conv_attn_layer = SeparableConv1D(embed_dim, self.all_head_size, self.conv_kernel_size)
            self.conv_kernel_layer = nn.Linear(self.all_head_size, self.num_heads * self.conv_kernel_size)
            self.conv_out_layer = nn.Linear(embed_dim, self.all_head_size)
            self.padding = (self.conv_kernel_size - 1) // 2

    def forward(self, query, key=None, value=None, attn_mask=None, cache=None):
        key = query if key is None else key
        value = query if value is None else value

        q = self.q_proj(query)
        k = self.k_proj(key)
        v = self.v_proj(value)

        if self.conv_type == "sdconv":
            bs = paddle.shape(q)[0]
            seqlen = paddle.shape(q)[1]
            mixed_key_conv_attn_layer = self.key_conv_attn_layer(query)
            conv_attn_layer = mixed_key_conv_attn_layer * q

            # conv_kernel_layer
            conv_kernel_layer = self.conv_kernel_layer(conv_attn_layer)
            conv_kernel_layer = tensor.reshape(conv_kernel_layer, shape=[-1, self.conv_kernel_size, 1])
            conv_kernel_layer = F.softmax(conv_kernel_layer, axis=1)
            conv_out_layer = self.conv_out_layer(query)
            conv_out_layer = F.pad(conv_out_layer, pad=[self.padding, self.padding], data_format="NLC")
            conv_out_layer = paddle.stack(
                [
                    paddle.slice(conv_out_layer, axes=[1], starts=[i], ends=[i + seqlen])
                    for i in range(self.conv_kernel_size)
                ],
                axis=-1,
            )
            conv_out_layer = tensor.reshape(conv_out_layer, shape=[-1, self.head_dim, self.conv_kernel_size])
            conv_out_layer = tensor.matmul(conv_out_layer, conv_kernel_layer)
            conv_out = tensor.reshape(conv_out_layer, shape=[bs, seqlen, self.num_heads, self.head_dim])

        q = tensor.reshape(x=q, shape=[0, 0, self.num_heads, self.head_dim])
        q = tensor.transpose(x=q, perm=[0, 2, 1, 3])
        k = tensor.reshape(x=k, shape=[0, 0, self.num_heads, self.head_dim])
        k = tensor.transpose(x=k, perm=[0, 2, 1, 3])
        v = tensor.reshape(x=v, shape=[0, 0, self.num_heads, self.head_dim])
        v = tensor.transpose(x=v, perm=[0, 2, 1, 3])

        product = tensor.matmul(x=q, y=k, transpose_y=True) * self.scale
        if attn_mask is not None:
            attn_mask = _convert_attention_mask(attn_mask, product.dtype)
            product = product + attn_mask

        weights = F.softmax(product)
        weights = self.dropout(weights)
        out = tensor.matmul(weights, v)

        # combine heads
        out = tensor.transpose(out, perm=[0, 2, 1, 3])
        if self.conv_type == "sdconv":
            out = tensor.concat([out, conv_out], axis=2)
        out = tensor.reshape(x=out, shape=[0, 0, out.shape[2] * out.shape[3]])

        # project to output
        out = self.out_proj(out)

        outs = [out]
        if self.need_weights:
            outs.append(weights)
        if cache is not None:
            outs.append(cache)
        return out if len(outs) == 1 else tuple(outs)


class TransformerEncoderLayerWithConv(nn.TransformerEncoderLayer):
    def __init__(
        self,
        d_model,
        nhead,
        dim_feedforward,
        dropout=0.1,
        activation="relu",
        attn_dropout=None,
        act_dropout=None,
        normalize_before=False,
        conv_kernel_size=None,
        head_ratio=None,
        num_groups=None,
        **kwargs
    ):
        super().__init__(
            d_model,
            nhead,
            dim_feedforward,
            dropout=dropout,
            activation=activation,
            attn_dropout=attn_dropout,
            act_dropout=act_dropout,
            normalize_before=normalize_before,
        )
        self.self_attn = MultiHeadAttentionWithConv(
            d_model,
            nhead,
            dropout=attn_dropout,
            conv_kernel_size=conv_kernel_size,
            head_ratio=head_ratio,
        )
        if num_groups > 1:
            self.linear1 = GroupedLinear(d_model, dim_feedforward, num_groups=num_groups)
            self.linear2 = GroupedLinear(dim_feedforward, d_model, num_groups=num_groups)
        self._config.update({"conv_kernel_size": conv_kernel_size, "head_ratio": head_ratio, "num_groups": num_groups})


class ConvBertEmbeddings(nn.Layer):
    """
    Include embeddings from word, position and token_type embeddings
    """

    def __init__(
        self,
        vocab_size,
        embedding_size,
        hidden_dropout_prob,
        max_position_embeddings,
        type_vocab_size,
    ):
        super(ConvBertEmbeddings, self).__init__()
        self.word_embeddings = nn.Embedding(vocab_size, embedding_size)
        self.position_embeddings = nn.Embedding(max_position_embeddings, embedding_size)
        self.token_type_embeddings = nn.Embedding(type_vocab_size, embedding_size)

        self.layer_norm = nn.LayerNorm(embedding_size, epsilon=1e-12)
        self.dropout = nn.Dropout(hidden_dropout_prob)

    def forward(self, input_ids, token_type_ids=None, position_ids=None):
        if position_ids is None:
            ones = paddle.ones_like(input_ids, dtype="int64")
            seq_length = paddle.cumsum(ones, axis=-1)
            position_ids = seq_length - ones
            position_ids.stop_gradient = True

        if token_type_ids is None:
            token_type_ids = paddle.zeros_like(input_ids, dtype="int64")

        input_embeddings = self.word_embeddings(input_ids)
        position_embeddings = self.position_embeddings(position_ids)
        token_type_embeddings = self.token_type_embeddings(token_type_ids)

        embeddings = input_embeddings + position_embeddings + token_type_embeddings
        embeddings = self.layer_norm(embeddings)
        embeddings = self.dropout(embeddings)
        return embeddings


class ConvBertDiscriminatorPredictions(nn.Layer):
    """
    Prediction layer for the discriminator.
    """

    def __init__(self, hidden_size, hidden_act):
        super(ConvBertDiscriminatorPredictions, self).__init__()

        self.dense = nn.Linear(hidden_size, hidden_size)
        self.dense_prediction = nn.Linear(hidden_size, 1)
        self.act = get_activation(hidden_act)

    def forward(self, discriminator_hidden_states):
        hidden_states = self.dense(discriminator_hidden_states)
        hidden_states = self.act(hidden_states)
        logits = self.dense_prediction(hidden_states).squeeze()

        return logits


class ConvBertGeneratorPredictions(nn.Layer):
    """
    Prediction layer for the generator.
    """

    def __init__(self, embedding_size, hidden_size, hidden_act):
        super(ConvBertGeneratorPredictions, self).__init__()

        self.layer_norm = nn.LayerNorm(embedding_size)
        self.dense = nn.Linear(hidden_size, embedding_size)
        self.act = get_activation(hidden_act)

    def forward(self, generator_hidden_states):
        hidden_states = self.dense(generator_hidden_states)
        hidden_states = self.act(hidden_states)
        hidden_states = self.layer_norm(hidden_states)

        return hidden_states


class ConvBertPretrainedModel(PretrainedModel):
    """
    An abstract class for pretrained ConvBert models. It provides ConvBert related
    `model_config_file`, `pretrained_init_configuration`, `resource_files_names`,
    `pretrained_resource_files_map`, `base_model_prefix` for downloading and
    loading pretrained models.
    See :class:`~paddlenlp.transformers.model_utils.PretrainedModel` for more details.
    """

    base_model_prefix = "convbert"

    # pretrained general configuration
    gen_weight = 1.0
    disc_weight = 50.0
    tie_word_embeddings = True
    untied_generator_embeddings = False
    use_softmax_sample = True

    # model init configuration
    pretrained_init_configuration = {
        "convbert-base": {
            "attention_probs_dropout_prob": 0.1,
            "embedding_size": 768,
            "hidden_act": "gelu",
            "hidden_dropout_prob": 0.1,
            "hidden_size": 768,
            "initializer_range": 0.02,
            "intermediate_size": 3072,
            "max_position_embeddings": 512,
            "num_attention_heads": 12,
            "num_hidden_layers": 12,
            "pad_token_id": 0,
            "type_vocab_size": 2,
            "vocab_size": 30522,
            "conv_kernel_size": 9,
            "head_ratio": 2,
            "num_groups": 1,
        },
        "convbert-medium-small": {
            "attention_probs_dropout_prob": 0.1,
            "embedding_size": 128,
            "hidden_act": "gelu",
            "hidden_dropout_prob": 0.1,
            "hidden_size": 384,
            "initializer_range": 0.02,
            "intermediate_size": 1536,
            "max_position_embeddings": 512,
            "num_attention_heads": 8,
            "num_hidden_layers": 12,
            "pad_token_id": 0,
            "type_vocab_size": 2,
            "vocab_size": 30522,
            "conv_kernel_size": 9,
            "head_ratio": 2,
            "num_groups": 2,
        },
        "convbert-small": {
            "attention_probs_dropout_prob": 0.1,
            "embedding_size": 128,
            "hidden_act": "gelu",
            "hidden_dropout_prob": 0.1,
            "hidden_size": 256,
            "initializer_range": 0.02,
            "intermediate_size": 1024,
            "max_position_embeddings": 512,
            "num_attention_heads": 4,
            "num_hidden_layers": 12,
            "pad_token_id": 0,
            "type_vocab_size": 2,
            "vocab_size": 30522,
            "conv_kernel_size": 9,
            "head_ratio": 2,
            "num_groups": 1,
        },
    }
    pretrained_resource_files_map = {
        "model_state": {
            "convbert-base": "http://bj.bcebos.com/paddlenlp/models/transformers/convbert/convbert-base/model_state.pdparams",
            "convbert-medium-small": "http://bj.bcebos.com/paddlenlp/models/transformers/convbert/convbert-medium-small/model_state.pdparams",
            "convbert-small": "http://bj.bcebos.com/paddlenlp/models/transformers/convbert/convbert-small/model_state.pdparams",
        }
    }

    def init_weights(self):
        """
        Initializes and tie weights if needed.
        """
        # Initialize weights
        self.apply(self._init_weights)
        # Tie weights if needed
        self.tie_weights()

    def tie_weights(self):
        """
        Tie the weights between the input embeddings and the output embeddings.
        """
        if hasattr(self, "get_output_embeddings") and hasattr(self, "get_input_embeddings"):
            output_embeddings = self.get_output_embeddings()
            if output_embeddings is not None:
                self._tie_or_clone_weights(output_embeddings, self.get_input_embeddings())

    def _init_weights(self, layer):
        """Initialize the weights"""
        if isinstance(layer, (nn.Linear, nn.Embedding, GroupedLinear)):
            layer.weight.set_value(
                paddle.tensor.normal(
                    mean=0.0,
                    std=self.initializer_range
                    if hasattr(self, "initializer_range")
                    else self.convbert.config["initializer_range"],
                    shape=layer.weight.shape,
                )
            )
        elif isinstance(layer, nn.LayerNorm):
            layer.bias.set_value(paddle.zeros_like(layer.bias))
            layer.weight.set_value(paddle.full_like(layer.weight, 1.0))
            layer._epsilon = 1e-12
        elif isinstance(layer, SeparableConv1D):
            layer.depthwise.weight.set_value(
                paddle.tensor.normal(
                    mean=0.0,
                    std=self.initializer_range
                    if hasattr(self, "initializer_range")
                    else self.convbert.config["initializer_range"],
                    shape=layer.depthwise.weight.shape,
                )
            )
            layer.pointwise.weight.set_value(
                paddle.tensor.normal(
                    mean=0.0,
                    std=self.initializer_range
                    if hasattr(self, "initializer_range")
                    else self.convbert.config["initializer_range"],
                    shape=layer.pointwise.weight.shape,
                )
            )

        if isinstance(layer, (nn.Linear, GroupedLinear, SeparableConv1D)) and layer.bias is not None:
            layer.bias.set_value(paddle.zeros_like(layer.bias))

    def _tie_or_clone_weights(self, output_embeddings, input_embeddings):
        """Tie or clone layer weights"""
        if output_embeddings.weight.shape == input_embeddings.weight.shape:
            output_embeddings.weight = input_embeddings.weight
        elif output_embeddings.weight.shape == input_embeddings.weight.t().shape:
            output_embeddings.weight.set_value(input_embeddings.weight.t())
        else:
            raise ValueError(
                "when tie input/output embeddings, the shape of output embeddings: {}"
                "should be equal to shape of input embeddings: {}"
                "or should be equal to the shape of transpose input embeddings: {}".format(
                    output_embeddings.weight.shape,
                    input_embeddings.weight.shape,
                    input_embeddings.weight.t().shape,
                )
            )
        if getattr(output_embeddings, "bias", None) is not None:
            if output_embeddings.weight.shape[-1] != output_embeddings.bias.shape[0]:
                raise ValueError(
                    "the weight lase shape: {} of output_embeddings is not equal to the bias shape: {}"
                    "please check output_embeddings configuration".format(
                        output_embeddings.weight.shape[-1],
                        output_embeddings.bias.shape[0],
                    )
                )


@register_base_model
class ConvBertModel(ConvBertPretrainedModel):
    """
    The bare ConvBert Model transformer outputting raw hidden-states.

    This model inherits from :class:`~paddlenlp.transformers.model_utils.PretrainedModel`.
    Refer to the superclass documentation for the generic methods.

    This model is also a Paddle `paddle.nn.Layer <https://www.paddlepaddle.org.cn/documentation
    /docs/en/api/paddle/fluid/dygraph/layers/Layer_en.html>`__ subclass. Use it as a regular Paddle Layer
    and refer to the Paddle documentation for all matter related to general usage and behavior.

    Args:
        vocab_size (int):
            Vocabulary size of `inputs_ids` in `ConvBertModel`. Also is the vocab size of token embedding matrix.
            Defines the number of different tokens that can be represented by the `inputs_ids` passed when calling `ConvBertModel`.
        embedding_size (int, optional):
            Dimensionality of the embedding layer. Defaults to `768`.
        hidden_size (int, optional):
            Dimensionality of the encoder layer and pooler layer. Defaults to `768`.
        num_hidden_layers (int, optional):
            Number of hidden layers in the Transformer encoder. Defaults to `12`.
        num_attention_heads (int, optional):
            Number of attention heads for each attention layer in the Transformer encoder.
            Defaults to `12`.
        intermediate_size (int, optional):
            Dimensionality of the feed-forward (ff) layer in the encoder. Input tensors
            to ff layers are firstly projected from `hidden_size` to `intermediate_size`,
            and then projected back to `hidden_size`. Typically `intermediate_size` is larger than `hidden_size`.
            Defaults to `3072`.
        hidden_act (str, optional):
            The non-linear activation function in the feed-forward layer.
            ``"gelu"``, ``"relu"`` and any other paddle supported activation functions
            are supported. Defaults to `"gelu"`.
        hidden_dropout_prob (float, optional):
            The dropout probability for all fully connected layers in the embeddings and encoder.
            Defaults to `0.1`.
        attention_probs_dropout_prob (float, optional):
            The dropout probability used in MultiHeadAttention in all encoder layers to drop some attention target.
            Defaults to `0.1`.
        max_position_embeddings (int, optional):
            The maximum value of the dimensionality of position encoding, which dictates the maximum supported length of an input
            sequence. Defaults to `512`.
        type_vocab_size (int, optional):
            The vocabulary size of `token_type_ids`.
            Defaults to `2`.

        initializer_range (float, optional):
            The standard deviation of the normal initializer.
            Defaults to 0.02.

            .. note::
                A normal_initializer initializes weight matrices as normal distributions.
                See :meth:`ConvBertPretrainedModel.init_weights()` for how weights are initialized in `ConvBertModel`.

        pad_token_id (int, optional):
            The index of padding token in the token vocabulary.
            Defaults to `0`.

        conv_kernel_size (int, optional):
            The size of the convolutional kernel.
            Defaults to `9`.

        head_ratio (int, optional):
            Ratio gamma to reduce the number of attention heads.
            Defaults to `2`.

        num_groups (int, optional):
            The number of groups for grouped linear layers for ConvBert model.
            Defaults to `1`.

    """

    def __init__(
        self,
        vocab_size,
        embedding_size=768,
        hidden_size=768,
        num_hidden_layers=12,
        num_attention_heads=12,
        intermediate_size=3072,
        hidden_act="gelu",
        hidden_dropout_prob=0.1,
        attention_probs_dropout_prob=0.1,
        max_position_embeddings=512,
        type_vocab_size=2,
        initializer_range=0.02,
        pad_token_id=0,
        conv_kernel_size=9,
        head_ratio=2,
        num_groups=1,
    ):
        super(ConvBertModel, self).__init__()
        self.pad_token_id = pad_token_id
        self.initializer_range = initializer_range
        self.embeddings = ConvBertEmbeddings(
            vocab_size,
            embedding_size,
            hidden_dropout_prob,
            max_position_embeddings,
            type_vocab_size,
        )

        if embedding_size != hidden_size:
            self.embeddings_project = nn.Linear(embedding_size, hidden_size)

        encoder_layer = TransformerEncoderLayerWithConv(
            hidden_size,
            num_attention_heads,
            intermediate_size,
            dropout=hidden_dropout_prob,
            activation=hidden_act,
            attn_dropout=attention_probs_dropout_prob,
            act_dropout=0,
            conv_kernel_size=conv_kernel_size,
            head_ratio=head_ratio,
            num_groups=num_groups,
        )
        self.encoder = nn.TransformerEncoder(encoder_layer, num_hidden_layers)

        self.init_weights()

    def get_input_embeddings(self):
        return self.embeddings.word_embeddings

    def set_input_embeddings(self, value):
        self.embeddings.word_embeddings = value

    def forward(self, input_ids, token_type_ids=None, position_ids=None, attention_mask=None):
        r"""
        The ConvBertModel forward method, overrides the `__call__()` special method.

        Args:
            input_ids (Tensor):
                Indices of input sequence tokens in the vocabulary. They are
                numerical representations of tokens that build the input sequence.
                Its data type should be `int64` and it has a shape of [batch_size, sequence_length].
            token_type_ids (Tensor, optional):
                Segment token indices to indicate different portions of the inputs.
                Selected in the range ``[0, type_vocab_size - 1]``.
                If `type_vocab_size` is 2, which means the inputs have two portions.
                Indices can either be 0 or 1:

                - 0 corresponds to a *sentence A* token,
                - 1 corresponds to a *sentence B* token.

                Its data type should be `int64` and it has a shape of [batch_size, sequence_length].
                Defaults to `None`, which means we don't add segment embeddings.
            position_ids(Tensor, optional):
                Indices of positions of each input sequence tokens in the position embeddings. Selected in the range ``[0,
                max_position_embeddings - 1]``.
                Shape as `(batch_size, num_tokens)` and dtype as int64. Defaults to `None`.
            attention_mask (Tensor, optional):
                Mask used in multi-head attention to avoid performing attention on to some unwanted positions,
                usually the paddings or the subsequent positions.
                Its data type can be int, float and bool.
                If its data type is int, the values should be either 0 or 1.

                - **1** for tokens that **not masked**,
                - **0** for tokens that **masked**.

                It is a tensor with shape broadcasted to `[batch_size, num_attention_heads, sequence_length, sequence_length]`.
                Defaults to `None`, which means nothing needed to be prevented attention to.

        Returns:
            Tensor: Returns Tensor `sequence_output`, sequence of hidden-states at the last layer of the model.
            Shape as `[batch_size, sequence_length, hidden_size]` and dtype as float32.

        Example:
            .. code-block::

                import paddle
                from paddlenlp.transformers import ConvBertModel, ConvBertTokenizer

                tokenizer = ConvBertTokenizer.from_pretrained('convbert-base')
                model = ConvBertModel.from_pretrained('convbert-base')

                inputs = tokenizer("Welcome to use PaddlePaddle and PaddleNLP!")
                inputs = {k:paddle.to_tensor([v]) for (k, v) in inputs.items()}
                output = model(**inputs)
        """
        if attention_mask is None:
            attention_mask = paddle.unsqueeze((input_ids == self.pad_token_id).astype(dtype_float) * -1e4, axis=[1, 2])
        else:
            attention_mask = paddle.unsqueeze(attention_mask, axis=[1, 2]).astype(dtype_float)
            attention_mask = (1.0 - attention_mask) * -1e4

        embedding_output = self.embeddings(
            input_ids=input_ids,
            position_ids=position_ids,
            token_type_ids=token_type_ids,
        )

        if hasattr(self, "embeddings_project"):
            embedding_output = self.embeddings_project(embedding_output)

        sequence_output = self.encoder(embedding_output, attention_mask)

        return sequence_output


class ConvBertDiscriminator(ConvBertPretrainedModel):
    """
    ConvBert Model with a discriminator prediction head on top.

    Args:
        convbert (:class:`ConvBertModel`):
            An instance of ConvBertModel.
    """

    def __init__(self, convbert):
        super(ConvBertDiscriminator, self).__init__()

        self.convbert = convbert
        self.discriminator_predictions = ConvBertDiscriminatorPredictions(
            self.convbert.config["hidden_size"], self.convbert.config["hidden_act"]
        )
        self.init_weights()

    def forward(self, input_ids, token_type_ids=None, position_ids=None, attention_mask=None):
        r"""
        The ConvBertDiscriminator forward method, overrides the `__call__()` special method.

        Args:
            input_ids (Tensor):
                Indices of input sequence tokens in the vocabulary. They are
                numerical representations of tokens that build the input sequence.
                Its data type should be `int64` and it has a shape of [batch_size, sequence_length].
            token_type_ids (Tensor, optional):
                Segment token indices to indicate different portions of the inputs.
                Selected in the range ``[0, type_vocab_size - 1]``.
                If `type_vocab_size` is 2, which means the inputs have two portions.
                Indices can either be 0 or 1:

                - 0 corresponds to a *sentence A* token,
                - 1 corresponds to a *sentence B* token.

                Its data type should be `int64` and it has a shape of [batch_size, sequence_length].
                Defaults to `None`, which means we don't add segment embeddings.
            position_ids(Tensor, optional):
                Indices of positions of each input sequence tokens in the position embeddings. Selected in the range ``[0,
                max_position_embeddings - 1]``.
                Shape as `(batch_size, num_tokens)` and dtype as int64. Defaults to `None`.
            attention_mask (Tensor, optional):
                Mask used in multi-head attention to avoid performing attention on to some unwanted positions,
                usually the paddings or the subsequent positions.
                Its data type can be int, float and bool.
                If its data type is int, the values should be either 0 or 1.

                - **1** for tokens that **not masked**,
                - **0** for tokens that **masked**.

                It is a tensor with shape broadcasted to `[batch_size, num_attention_heads, sequence_length, sequence_length]`.
                Defaults to `None`, which means nothing needed to be prevented attention to.

        Returns:
            Tensor: Returns tensor `logits`, a tensor of the discriminator prediction logits.
            Shape as `[batch_size, sequence_length]` and dtype as float32.

        Example:
            .. code-block::

                import paddle
                from paddlenlp.transformers import ConvBertDiscriminatorPredictions, ConvBertTokenizer

                tokenizer = ConvBertTokenizer.from_pretrained('convbert-base')
                model = ConvBertDiscriminatorPredictions.from_pretrained('convbert-base')

                inputs = tokenizer("Welcome to use PaddlePaddle and PaddleNLP!")
                inputs = {k:paddle.to_tensor([v]) for (k, v) in inputs.items()}
                logits = model(**inputs)
        """

        discriminator_sequence_output = self.convbert(input_ids, token_type_ids, position_ids, attention_mask)

        logits = self.discriminator_predictions(discriminator_sequence_output)

        return logits


class ConvBertGenerator(ConvBertPretrainedModel):
    """
    ConvBert Model with a generator prediction head on top.

    Args:
        convbert (:class:`ConvBertModel`):
            An instance of ConvBertModel.
    """

    def __init__(self, convbert):
        super(ConvBertGenerator, self).__init__()

        self.convbert = convbert
        self.generator_predictions = ConvBertGeneratorPredictions(
            self.convbert.config["embedding_size"],
            self.convbert.config["hidden_size"],
            self.convbert.config["hidden_act"],
        )

        if not self.tie_word_embeddings:
            self.generator_lm_head = nn.Linear(
                self.convbert.config["embedding_size"], self.convbert.config["vocab_size"]
            )
        else:
            self.generator_lm_head_bias = paddle.create_parameter(
                shape=[self.convbert.config["vocab_size"]],
                dtype=dtype_float,
                is_bias=True,
            )
        self.init_weights()

    def get_input_embeddings(self):
        return self.convbert.embeddings.word_embeddings

    def forward(
        self,
        input_ids=None,
        token_type_ids=None,
        position_ids=None,
        attention_mask=None,
    ):
        r"""
        The ConvBertGenerator forward method, overrides the `__call__()` special method.

        Args:
            input_ids (Tensor):
                Indices of input sequence tokens in the vocabulary. They are
                numerical representations of tokens that build the input sequence.
                Its data type should be `int64` and it has a shape of [batch_size, sequence_length].
            token_type_ids (Tensor, optional):
                Segment token indices to indicate different portions of the inputs.
                Selected in the range ``[0, type_vocab_size - 1]``.
                If `type_vocab_size` is 2, which means the inputs have two portions.
                Indices can either be 0 or 1:

                - 0 corresponds to a *sentence A* token,
                - 1 corresponds to a *sentence B* token.

                Its data type should be `int64` and it has a shape of [batch_size, sequence_length].
                Defaults to `None`, which means we don't add segment embeddings.
            position_ids(Tensor, optional):
                Indices of positions of each input sequence tokens in the position embeddings. Selected in the range ``[0,
                max_position_embeddings - 1]``.
                Shape as `(batch_size, num_tokens)` and dtype as int64. Defaults to `None`.
            attention_mask (Tensor, optional):
                Mask used in multi-head attention to avoid performing attention on to some unwanted positions,
                usually the paddings or the subsequent positions.
                Its data type can be int, float and bool.
                If its data type is int, the values should be either 0 or 1.

                - **1** for tokens that **not masked**,
                - **0** for tokens that **masked**.

                It is a tensor with shape broadcasted to `[batch_size, num_attention_heads, sequence_length, sequence_length]`.
                Defaults to `None`, which means nothing needed to be prevented attention to.

        Returns:
            Tensor: Returns tensor `prediction_scores`, a tensor of the generator prediction scores.
            Shape as `[batch_size, sequence_length, vocab_size]` and dtype as float32.

        Example:
            .. code-block::

                import paddle
                from paddlenlp.transformers import ConvBertGenerator, ConvBertTokenizer

                tokenizer = ConvBertTokenizer.from_pretrained('convbert-base')
                model = ConvBertGenerator.from_pretrained('convbert-base')

                inputs = tokenizer("Welcome to use PaddlePaddle and PaddleNLP!")
                inputs = {k:paddle.to_tensor([v]) for (k, v) in inputs.items()}
                prediction_scores = model(**inputs)
        """

        generator_sequence_output = self.convbert(input_ids, token_type_ids, position_ids, attention_mask)

        prediction_scores = self.generator_predictions(generator_sequence_output)
        if not self.tie_word_embeddings:
            prediction_scores = self.generator_lm_head(prediction_scores)
        else:
            prediction_scores = paddle.add(
                paddle.matmul(
                    prediction_scores,
                    self.get_input_embeddings().weight,
                    transpose_y=True,
                ),
                self.generator_lm_head_bias,
            )

        return prediction_scores


class ConvBertClassificationHead(nn.Layer):
    """
    ConvBert head for sentence-level classification tasks.
    """

    def __init__(self, hidden_size, hidden_dropout_prob, num_classes):
        super(ConvBertClassificationHead, self).__init__()
        self.dense = nn.Linear(hidden_size, hidden_size)
        self.dropout = nn.Dropout(hidden_dropout_prob)
        self.out_proj = nn.Linear(hidden_size, num_classes)
        self.act = get_activation("gelu")

    def forward(self, features, **kwargs):
        x = features[:, 0, :]  # take [CLS] token
        x = self.dropout(x)
        x = self.dense(x)
        x = self.act(x)  # ConvBert paper used gelu here
        x = self.dropout(x)
        x = self.out_proj(x)
        return x


class ConvBertForSequenceClassification(ConvBertPretrainedModel):
    """
    ConvBert Model with a linear layer on top of the output layer,
    designed for sequence classification/regression tasks like GLUE tasks.

    Args:
        convbert (:class:`ConvBertModel`):
            An instance of ConvBertModel.
        num_classes (int, optional):
            The number of classes. Defaults to `2`.
        dropout (float, optional):
            The dropout probability for output of ConvBert.
            If None, use the same value as `hidden_dropout_prob` of `ConvBertModel`
            instance `convbert`. Defaults to None.
    """

    def __init__(self, convbert, num_classes=2, dropout=None):
        super(ConvBertForSequenceClassification, self).__init__()
        self.num_classes = num_classes
        self.convbert = convbert
        self.classifier = ConvBertClassificationHead(
            hidden_size=self.convbert.config["hidden_size"],
            hidden_dropout_prob=dropout if dropout is not None else self.convbert.config["hidden_dropout_prob"],
            num_classes=self.num_classes,
        )

        self.init_weights()

    def forward(
        self,
        input_ids=None,
        token_type_ids=None,
        position_ids=None,
        attention_mask=None,
    ):
        r"""
        The ConvBertForSequenceClassification forward method, overrides the __call__() special method.

        Args:
            input_ids (Tensor):
                See :class:`ConvBertModel`.
            token_type_ids (Tensor, optional):
                See :class:`ConvBertModel`.
            position_ids(Tensor, optional):
                See :class:`ConvBertModel`.
            attention_mask (list, optional):
                See :class:`ConvBertModel`.

        Returns:
            Tensor: Returns tensor `logits`, a tensor of the input text classification logits.
            Shape as `[batch_size, num_classes]` and dtype as float32.

        Example:
            .. code-block::

                import paddle
                from paddlenlp.transformers import ConvBertForSequenceClassification, ConvBertTokenizer

                tokenizer = ConvBertTokenizer.from_pretrained('convbert-base')
                model = ConvBertForSequenceClassification.from_pretrained('convbert-base')

                inputs = tokenizer("Welcome to use PaddlePaddle and PaddleNLP!")
                inputs = {k:paddle.to_tensor([v]) for (k, v) in inputs.items()}
                logits = model(**inputs)
        """

        sequence_output = self.convbert(input_ids, token_type_ids, position_ids, attention_mask)

        logits = self.classifier(sequence_output)

        return logits


class ConvBertForTokenClassification(ConvBertPretrainedModel):
    """
    ConvBert Model with a linear layer on top of the hidden-states output layer,
    designed for token classification tasks like NER tasks.

    Args:
        convbert (:class:`ConvBertModel`):
            An instance of ConvBertModel.
        num_classes (int, optional):
            The number of classes. Defaults to `2`.
        dropout (float, optional):
            The dropout probability for output of ConvBert.
            If None, use the same value as `hidden_dropout_prob` of `ConvBertModel`
            instance `convbert`. Defaults to None.
    """

    def __init__(self, convbert, num_classes=2, dropout=None):
        super(ConvBertForTokenClassification, self).__init__()
        self.num_classes = num_classes
        self.convbert = convbert
        self.dropout = nn.Dropout(dropout if dropout is not None else self.convbert.config["hidden_dropout_prob"])
        self.classifier = nn.Linear(self.convbert.config["hidden_size"], self.num_classes)
        self.init_weights()

    def forward(
        self,
        input_ids=None,
        token_type_ids=None,
        position_ids=None,
        attention_mask=None,
    ):
        r"""
        The ConvBertForTokenClassification forward method, overrides the __call__() special method.

        Args:
            input_ids (Tensor):
                See :class:`ConvBertModel`.
            token_type_ids (Tensor, optional):
                See :class:`ConvBertModel`.
            position_ids(Tensor, optional):
                See :class:`ConvBertModel`.
            attention_mask (list, optional):
                See :class:`ConvBertModel`.

        Returns:
            Tensor: Returns tensor `logits`, a tensor of the input token classification logits.
            Shape as `[batch_size, sequence_length, num_classes]` and dtype as `float32`.

        Example:
            .. code-block::

                import paddle
                from paddlenlp.transformers import ConvBertForTokenClassification, ConvBertTokenizer

                tokenizer = ConvBertTokenizer.from_pretrained('convbert-base')
                model = ConvBertForTokenClassification.from_pretrained('convbert-base')

                inputs = tokenizer("Welcome to use PaddlePaddle and PaddleNLP!")
                inputs = {k:paddle.to_tensor([v]) for (k, v) in inputs.items()}
                logits = model(**inputs)
        """

        sequence_output = self.convbert(input_ids, token_type_ids, position_ids, attention_mask)

        sequence_output = self.dropout(sequence_output)
        logits = self.classifier(sequence_output)

        return logits


class ConvBertForTotalPretraining(ConvBertPretrainedModel):
    """
    Combine generator with discriminator for Replaced Token Detection (RTD) pretraining.
    """

    pretrained_init_configuration = {
        "convbert-base-generator": {
            "attention_probs_dropout_prob": 0.1,
            "embedding_size": 768,
            "hidden_act": "gelu",
            "hidden_dropout_prob": 0.1,
            "hidden_size": 256,
            "initializer_range": 0.02,
            "intermediate_size": 1024,
            "max_position_embeddings": 512,
            "num_attention_heads": 4,
            "num_hidden_layers": 12,
            "pad_token_id": 0,
            "type_vocab_size": 2,
            "vocab_size": 30522,
            "conv_kernel_size": 9,
            "head_ratio": 2,
            "num_groups": 1,
        },
        "convbert-medium-small-generator": {
            "attention_probs_dropout_prob": 0.1,
            "embedding_size": 128,
            "hidden_act": "gelu",
            "hidden_dropout_prob": 0.1,
            "hidden_size": 96,
            "initializer_range": 0.02,
            "intermediate_size": 384,
            "max_position_embeddings": 512,
            "num_attention_heads": 2,
            "num_hidden_layers": 12,
            "pad_token_id": 0,
            "type_vocab_size": 2,
            "vocab_size": 30522,
            "conv_kernel_size": 9,
            "head_ratio": 2,
            "num_groups": 2,
        },
        "convbert-small-generator": {
            "attention_probs_dropout_prob": 0.1,
            "embedding_size": 128,
            "hidden_act": "gelu",
            "hidden_dropout_prob": 0.1,
            "hidden_size": 64,
            "initializer_range": 0.02,
            "intermediate_size": 256,
            "max_position_embeddings": 512,
            "num_attention_heads": 1,
            "num_hidden_layers": 12,
            "pad_token_id": 0,
            "type_vocab_size": 2,
            "vocab_size": 30522,
            "conv_kernel_size": 9,
            "head_ratio": 2,
            "num_groups": 1,
        },
        "convbert-base-discriminator": {
            "attention_probs_dropout_prob": 0.1,
            "embedding_size": 768,
            "hidden_act": "gelu",
            "hidden_dropout_prob": 0.1,
            "hidden_size": 768,
            "initializer_range": 0.02,
            "intermediate_size": 3072,
            "max_position_embeddings": 512,
            "num_attention_heads": 12,
            "num_hidden_layers": 12,
            "pad_token_id": 0,
            "type_vocab_size": 2,
            "vocab_size": 30522,
            "conv_kernel_size": 9,
            "head_ratio": 2,
            "num_groups": 1,
        },
        "convbert-medium-small-discriminator": {
            "attention_probs_dropout_prob": 0.1,
            "embedding_size": 128,
            "hidden_act": "gelu",
            "hidden_dropout_prob": 0.1,
            "hidden_size": 384,
            "initializer_range": 0.02,
            "intermediate_size": 1536,
            "max_position_embeddings": 512,
            "num_attention_heads": 8,
            "num_hidden_layers": 12,
            "pad_token_id": 0,
            "type_vocab_size": 2,
            "vocab_size": 30522,
            "conv_kernel_size": 9,
            "head_ratio": 2,
            "num_groups": 2,
        },
        "convbert-small-discriminator": {
            "attention_probs_dropout_prob": 0.1,
            "embedding_size": 128,
            "hidden_act": "gelu",
            "hidden_dropout_prob": 0.1,
            "hidden_size": 256,
            "initializer_range": 0.02,
            "intermediate_size": 1024,
            "max_position_embeddings": 512,
            "num_attention_heads": 4,
            "num_hidden_layers": 12,
            "pad_token_id": 0,
            "type_vocab_size": 2,
            "vocab_size": 30522,
            "conv_kernel_size": 9,
            "head_ratio": 2,
            "num_groups": 1,
        },
    }

    def __init__(self, generator, discriminator):
        super(ConvBertForTotalPretraining, self).__init__()

        self.generator = generator
        self.discriminator = discriminator
        self.initializer_range = discriminator.convbert.initializer_range
        self.init_weights()

    def get_input_embeddings(self):
        if not self.untied_generator_embeddings:
            return self.generator.convbert.embeddings.word_embeddings
        else:
            return None

    def get_output_embeddings(self):
        if not self.untied_generator_embeddings:
            return self.discriminator.convbert.embeddings.word_embeddings
        else:
            return None

    def get_discriminator_inputs(self, inputs, raw_inputs, gen_logits, gen_labels, use_softmax_sample):
        """Sample from the generator to create discriminator input."""
        # get generator token result
        sampled_tokens = (self.sample_from_softmax(gen_logits, use_softmax_sample)).detach()
        sampled_tokids = paddle.argmax(sampled_tokens, axis=-1)
        # update token only at mask position
        # gen_labels : [B, L], L contains -100(unmasked) or token value(masked)
        # mask_positions : [B, L], L contains 0(unmasked) or 1(masked)
        umask_positions = paddle.zeros_like(gen_labels)
        mask_positions = paddle.ones_like(gen_labels)
        mask_positions = paddle.where(gen_labels == -100, umask_positions, mask_positions)
        updated_inputs = self.update_inputs(inputs, sampled_tokids, mask_positions)
        # use inputs and updated_input to get discriminator labels
        labels = mask_positions * (paddle.ones_like(inputs) - paddle.equal(updated_inputs, raw_inputs).astype("int32"))
        return updated_inputs, labels, sampled_tokids

    def sample_from_softmax(self, logits, use_softmax_sample=True):
        if use_softmax_sample:
            # uniform_noise = paddle.uniform(logits.shape, dtype="float32", min=0, max=1)
            uniform_noise = paddle.rand(logits.shape, dtype=dtype_float)
            gumbel_noise = -paddle.log(-paddle.log(uniform_noise + 1e-9) + 1e-9)
        else:
            gumbel_noise = paddle.zeros_like(logits)
        # softmax_sample equal to sampled_tokids.unsqueeze(-1)
        softmax_sample = paddle.argmax(F.softmax(logits + gumbel_noise), axis=-1)
        # one hot
        return F.one_hot(softmax_sample, logits.shape[-1])

    def update_inputs(self, sequence, updates, positions):
        shape = sequence.shape
        assert len(shape) == 2, "the dimension of inputs should be [batch_size, sequence_length]"
        B, L = shape
        N = positions.shape[1]
        assert N == L, "the dimension of inputs and mask should be same as [batch_size, sequence_length]"

        updated_sequence = ((paddle.ones_like(sequence) - positions) * sequence) + (positions * updates)

        return updated_sequence

    def forward(
        self,
        input_ids=None,
        token_type_ids=None,
        position_ids=None,
        attention_mask=None,
        raw_input_ids=None,
        gen_labels=None,
    ):
        r"""

        Args:
            input_ids (Tensor):
                See :class:`ConvBertModel`.
            token_type_ids (Tensor, optional):
                See :class:`ConvBertModel`.
            position_ids (Tensor, optional):
                See :class:`ConvBertModel`.
            attention_mask (Tensor, optional):
                See :class:`ConvBertModel`.
            raw_input_ids(Tensor, optional):
                The raw input_ids. Its data type should be `int64` and it has a shape of [batch_size, sequence_length].
            gen_labels(Tensor, optional):
                The generator labels. Its data type should be `int64` and it has a shape of [batch_size, sequence_length].

        Returns:
            tuple: Returns tuple (``gen_logits``, ``disc_logits``, ``disc_labels``, ``attention_mask``).

            With the fields:

            - `gen_logits` (Tensor):
                a tensor of the generator prediction logits. Shape as `[batch_size, sequence_length, vocab_size]` and dtype as float32.

            - `disc_logits` (Tensor):
                a tensor of the discriminator prediction logits. Shape as `[batch_size, sequence_length]` and dtype as float32.

            - `disc_labels` (Tensor):
                a tensor of the discriminator prediction labels. Shape as `[batch_size, sequence_length]` and dtype as int64.

            - `attention_mask` (Tensor):
                See :class:`ConvBertModel`.
        """

        assert gen_labels is not None, "gen_labels should not be None"

        gen_logits = self.generator(input_ids, token_type_ids, position_ids, attention_mask)

        (
            disc_inputs,
            disc_labels,
            generator_predict_tokens,
        ) = self.get_discriminator_inputs(input_ids, raw_input_ids, gen_logits, gen_labels, self.use_softmax_sample)

        disc_logits = self.discriminator(disc_inputs, token_type_ids, position_ids, attention_mask)

        if attention_mask is None:
            attention_mask = input_ids != self.discriminator.convbert.config["pad_token_id"]
        else:
            attention_mask = attention_mask.astype("bool")

        return gen_logits, disc_logits, disc_labels, attention_mask


class ConvBertPretrainingCriterion(nn.Layer):
    """

    Args:
        vocab_size(int):
            Vocabulary size of `inputs_ids` in `ConvBertModel`. Defines the number of different tokens that can
            be represented by the `inputs_ids` passed when calling `ConvBertModel`.
        gen_weight(float):
            This is the generator weight.
        disc_weight(float):
            This is the discriminator weight.

    """

    def __init__(self, vocab_size, gen_weight, disc_weight):
        super(ConvBertPretrainingCriterion, self).__init__()

        self.vocab_size = vocab_size
        self.gen_weight = gen_weight
        self.disc_weight = disc_weight
        self.gen_loss_fct = nn.CrossEntropyLoss(reduction="none")
        self.disc_loss_fct = nn.BCEWithLogitsLoss(reduction="none")

    def forward(
        self,
        generator_prediction_scores,
        discriminator_prediction_scores,
        generator_labels,
        discriminator_labels,
        attention_mask,
    ):
        # generator loss
        gen_loss = self.gen_loss_fct(
            paddle.reshape(generator_prediction_scores, [-1, self.vocab_size]),
            paddle.reshape(generator_labels, [-1]),
        )
        # todo: we can remove 4 lines after when CrossEntropyLoss(reduction='mean') improved
        umask_positions = paddle.zeros_like(generator_labels).astype(dtype_float)
        mask_positions = paddle.ones_like(generator_labels).astype(dtype_float)
        mask_positions = paddle.where(generator_labels == -100, umask_positions, mask_positions)
        if mask_positions.sum() == 0:
            gen_loss = paddle.to_tensor([0.0])
        else:
            gen_loss = gen_loss.sum() / mask_positions.sum()

        # discriminator loss
        seq_length = discriminator_labels.shape[1]
        disc_loss = self.disc_loss_fct(
            paddle.reshape(discriminator_prediction_scores, [-1, seq_length]),
            discriminator_labels.astype(dtype_float),
        )
        if attention_mask is not None:
            umask_positions = paddle.ones_like(discriminator_labels).astype(dtype_float)
            mask_positions = paddle.zeros_like(discriminator_labels).astype(dtype_float)
            use_disc_loss = paddle.where(attention_mask, disc_loss, mask_positions)
            umask_positions = paddle.where(attention_mask, umask_positions, mask_positions)
            disc_loss = use_disc_loss.sum() / umask_positions.sum()
        else:
            total_positions = paddle.ones_like(discriminator_labels).astype(dtype_float)
            disc_loss = disc_loss.sum() / total_positions.sum()

        return self.gen_weight * gen_loss + self.disc_weight * disc_loss


class ConvBertPooler(Layer):
    def __init__(self, hidden_size, pool_act="tanh"):
        super(ConvBertPooler, self).__init__()
        self.dense = nn.Linear(hidden_size, hidden_size)
        self.activation = nn.Tanh()
        self.pool_act = pool_act

    def forward(self, hidden_states):
        # We "pool" the model by simply taking the hidden state corresponding
        # to the first token.
        first_token_tensor = hidden_states[:, 0]
        pooled_output = self.dense(first_token_tensor)
        if self.pool_act == "tanh":
            pooled_output = self.activation(pooled_output)
        return pooled_output


class ConvBertForMultipleChoice(ConvBertPretrainedModel):
    """
    ConvBert Model with a linear layer on top of the hidden-states output layer,
    designed for multiple choice tasks like RocStories/SWAG tasks .

    Args:
        convbert (:class:`ConvBertModel`):
            An instance of ConvBertModel.
        num_choices (int, optional):
            The number of choices. Defaults to `2`.
        dropout (float, optional):
            The dropout probability for output of ConvBert.
            If None, use the same value as `hidden_dropout_prob` of `ConvBertModel`
            instance `convbert`. Defaults to None.
    """

    def __init__(self, convbert, num_choices=2, dropout=None):
        super(ConvBertForMultipleChoice, self).__init__()
        self.num_choices = num_choices
        self.convbert = convbert
        self.pooler = ConvBertPooler(self.convbert.config["hidden_size"])
        self.dropout = nn.Dropout(dropout if dropout is not None else self.convbert.config["hidden_dropout_prob"])
        self.classifier = nn.Linear(self.convbert.config["hidden_size"], 1)
        self.init_weights()

    def forward(self, input_ids=None, token_type_ids=None, position_ids=None, attention_mask=None):
        r"""
        The ConvBertForMultipleChoice forward method, overrides the __call__() special method.

        Args:
            input_ids (Tensor):
                See :class:`ConvBertModel` and shape as [batch_size,num_choice, sequence_length].
            token_type_ids (Tensor, optional):
                See :class:`ConvBertModel` and shape as [batch_size,num_choice, sequence_length].
            position_ids(Tensor, optional):
                See :class:`ConvBertModel` and shape as [batch_size,num_choice, sequence_length].
            attention_mask (list, optional):
                See :class:`ConvBertModel` and shape as [batch_size,num_choice, sequence_length].

        Returns:
            Tensor: Returns tensor `reshaped_logits`, a tensor of the multiple choice classification logits.
            Shape as `[batch_size, num_choice]` and dtype as `float32`.

        Example:
            .. code-block::

                import paddle
                from paddlenlp.transformers import ConvBertForMultipleChoice, ConvBertTokenizer

                tokenizer = ConvBertTokenizer.from_pretrained('convbert-base')
                model = ConvBertForMultipleChoice.from_pretrained('convbert-base')

                inputs = tokenizer("Welcome to use PaddlePaddle and PaddleNLP!")
                inputs = {k:paddle.to_tensor([v]) for (k, v) in inputs.items()}
                logits = model(**inputs)
        """
        input_ids = input_ids.reshape((-1, input_ids.shape[-1]))  # flat_input_ids: [bs*num_choice,seq_l]

        if token_type_ids is not None:
            token_type_ids = token_type_ids.reshape((-1, token_type_ids.shape[-1]))
        if position_ids is not None:
            position_ids = position_ids.reshape((-1, position_ids.shape[-1]))
        if attention_mask is not None:
            attention_mask = attention_mask.reshape((-1, attention_mask.shape[-1]))

        sequence_output = self.convbert(input_ids, token_type_ids, position_ids, attention_mask)
        pooled_output = self.pooler(sequence_output)
        pooled_output = self.dropout(pooled_output)
        logits = self.classifier(pooled_output)  # logits: (bs*num_choice,1)
        reshaped_logits = logits.reshape((-1, self.num_choices))  # logits: (bs, num_choice)

        return reshaped_logits


class ConvBertForQuestionAnswering(ConvBertPretrainedModel):
    """
    ConvBert Model with a linear layer on top of the hidden-states output to compute `span_start_logits`
    and `span_end_logits`, designed for question-answering tasks like SQuAD.

    Args:
        convbert (:class:`ConvBertModel`):
            An instance of ConvBertModel.

    """

    def __init__(self, convbert):
        super(ConvBertForQuestionAnswering, self).__init__()
        self.convbert = convbert
        self.classifier = nn.Linear(self.convbert.config["hidden_size"], 2)
        self.init_weights()

    def forward(self, input_ids, token_type_ids=None, position_ids=None, attention_mask=None):
        r"""
        The ConvBertForQuestionAnswering forward method, overrides the __call__() special method.

        Args:
            input_ids (Tensor):
                See :class:`ConvBertModel`.
            token_type_ids (Tensor, optional):
                See :class:`ConvBertModel`.
            position_ids(Tensor, optional):
                See :class:`ConvBertModel`.
            attention_mask (list, optional):
                See :class:`ConvBertModel`.
        Returns:
            tuple: Returns tuple (`start_logits`, `end_logits`).

            With the fields:

            - `start_logits` (Tensor):
                A tensor of the input token classification logits, indicates the start position of the labelled span.
                Its data type should be float32 and its shape is [batch_size, sequence_length].

            - `end_logits` (Tensor):
                A tensor of the input token classification logits, indicates the end position of the labelled span.
                Its data type should be float32 and its shape is [batch_size, sequence_length].

        Example:
            .. code-block::

                import paddle
                from paddlenlp.transformers import ConvBertForQuestionAnswering, ConvBertTokenizer

                tokenizer = ConvBertTokenizer.from_pretrained('convbert-base')
                model = ConvBertForQuestionAnswering.from_pretrained('convbert-base')

                inputs = tokenizer("Welcome to use PaddlePaddle and PaddleNLP!")
                inputs = {k:paddle.to_tensor([v]) for (k, v) in inputs.items()}
                outputs = model(**inputs)

                start_logits = outputs[0]
                end_logits  = outputs[1]

        """
        sequence_output = self.convbert(
            input_ids, token_type_ids, position_ids=position_ids, attention_mask=attention_mask
        )
        logits = self.classifier(sequence_output)
        logits = paddle.transpose(logits, perm=[2, 0, 1])
        start_logits, end_logits = paddle.unstack(x=logits, axis=0)

        return start_logits, end_logits


ConvBertForPretraining = ConvBertForTotalPretraining



# Copyright (c) 2022 PaddlePaddle Authors. All Rights Reserved.
# Copyright 2021-2022 The Fairseq Authors and The Google Flax
# Team Authors And The HuggingFace Inc. team and & DALL·E Mini team.
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import html
import math
import random
import re
from pathlib import Path

from paddle.utils import try_import
from ...transformers import GPTTokenizer, AddedToken

__all__ = ["DalleBartTokenizer"]

PRETRAINED_POSITIONAL_EMBEDDINGS_SIZES = {
    "dalle-mini": 64,
    "dalle-mega-v16": 64,
    "dalle-mega-v26": 64,
    "dalle-mega": 64,
}

# based on wiki word occurrence
person_token = [("a person", 282265), ("someone", 121194), ("somebody", 12219)]
temp_token = "xtokx"  # avoid repeating chars


class HashtagProcessor:
    # Adapted from wordninja library
    # We use our wikipedia word count + a good heuristic to make it work
    def __init__(self, wiki_word_frequency):
        self._word_cost = (l.split()[0] for l in Path(wiki_word_frequency).read_text(encoding="utf8").splitlines())
        self._word_cost = {str(k): math.log(float(i + 1)) for i, k in enumerate(self._word_cost)}
        self._max_word = max(len(x) for x in self._word_cost.keys())
        self._SPLIT_RE = re.compile("[^a-zA-Z0-9']+")

    def __call__(self, s):
        """Uses dynamic programming to infer the location of spaces in a string without spaces."""
        l = [self._split(x) for x in self._SPLIT_RE.split(s)]
        return " ".join([item for sublist in l for item in sublist])

    def _split(self, s):
        # Find the best match for the i first characters, assuming cost has
        # been built for the i-1 first characters.
        # Returns a pair (match_cost, match_length).
        def best_match(i):
            candidates = enumerate(reversed(cost[max(0, i - self._max_word) : i]))
            return min((c + self._word_cost.get(s[i - k - 1 : i].lower(), 9e999), k + 1) for k, c in candidates)

        # Build the cost array
        cost = [0]
        for i in range(1, len(s) + 1):
            c, k = best_match(i)
            cost.append(c)

        # Backtrack to recover the minimal-cost string.
        out = []
        i = len(s)
        while i > 0:
            c, k = best_match(i)
            assert c == cost[i]
            newToken = True
            if not s[i - k : i] == "'":  # ignore a lone apostrophe
                if len(out) > 0:
                    # re-attach split 's and split digits
                    if out[-1] == "'s" or (s[i - 1].isdigit() and out[-1][0].isdigit()):  # digit followed by digit
                        out[-1] = s[i - k : i] + out[-1]  # combine current token with previous token
                        newToken = False

            if newToken:
                out.append(s[i - k : i])

            i -= k

        return reversed(out)


def replace_person_token(t):
    "Used for CC12M"
    t = re.sub(r"<person>([,\s]*(and)*[,\s]*<person>)+", " people ", t)
    while "<person>" in t:
        t = t.replace("<person>", f" {random.choices(*tuple(zip(*person_token)))[0]} ", 1)
    return t


def fix_html(t):
    # from OpenAI CLIP
    return html.unescape(html.unescape(t))


def replace_punctuation_with_commas(t):
    return re.sub(r"[()[\].,|:;?!=+~\-\/{}]", ",", t)


def simplify_quotes(t):
    return re.sub("""['"`]""", ' " ', t)


def merge_quotes(t):
    return re.sub(r'(\s*"+\s*)+', ' " ', t)


def remove_comma_numbers(t):
    def _f(t):
        return re.sub(r"(\d),(\d{3})", r"\1\2", t)

    return _f(_f(t))


def pre_process_dot_numbers(t):
    return re.sub(r"(\w)\.(\w)", rf"\1{temp_token}dot{temp_token}\2", t)


def post_process_dot_numbers(t):
    return re.sub(f"{temp_token}dot{temp_token}", ".", t)


def pre_process_quotes(t):
    # allows quotes only for 's, 't, 'd, 'm, 'll, 're, 've
    return re.sub(r"'(?=([stdm]|(ll)|(re)|(ve)|(ll))\b)", rf"{temp_token}quote{temp_token}", t)


def post_process_quotes(t):
    return re.sub(f"{temp_token}quote{temp_token}", "'", t)


def pre_process_dates(t):
    return re.sub(r"(\d)/(\d)", rf"\1{temp_token}slash{temp_token}\2", t)


def post_process_dates(t):
    return re.sub(f"{temp_token}slash{temp_token}", "/", t)


def merge_commas(t):
    return re.sub(r"(\s*,+\s*)+", ", ", t)


def add_space_after_commas(t):
    return re.sub(",", ", ", t)


def handle_special_chars(t):
    "Handle special characters"
    # replace "-" with a space when between words without space
    t = re.sub(r"(\w)-(\w)", r"\1 \2", t)
    # always add space around some characters
    return re.sub(r"([%&\/$*])", r" \1 ", t)


def expand_hashtags(t, hashtag_processor):
    "Remove # and try to split words"
    return re.sub(r"#(\w+)", lambda m: hashtag_processor(m.group(1)), t)


_re_ignore_chars = r"[_#\\]"


def ignore_chars(t):
    "Ignore useless characters"
    return re.sub(_re_ignore_chars, " ", t)


def remove_extra_spaces(t):
    "Remove extra spaces (including \t and \n)"
    return re.sub(r"\s+", " ", t)


def remove_repeating_chars(t):
    "If the same character is present 4+ times (not 3 because of roman 'VIII'), replace with single instance"
    return re.sub(r"(\D)(\1{3,})", r"\1", t)


def remove_urls(t):
    return re.sub(r"http\S+", "", t)


def remove_html_tags(t):
    return re.sub("<[^<]+?>", " ", t)


def remove_first_last_commas(t):
    t = t.strip()
    t = t[:-1] if t and t[-1] == "," else t
    t = t[1:] if t and t[0] == "," else t
    return t.strip()


def remove_wiki_ref(t):
    t = re.sub(r"\A\s*\[\d+\]", "", t)
    return re.sub(r"\[\d+\]\s*\Z", "", t)


class TextNormalizer:
    def __init__(self, wiki_word_frequency_file):
        self._hashtag_processor = HashtagProcessor(wiki_word_frequency_file)
        self.emoji = try_import("emoji")
        self.ftfy = try_import("ftfy")
        self.unidecode = try_import("unidecode")

    def __call__(self, t):
        # fix some characters
        t = self.ftfy.fix_text(t)
        # fix html
        t = fix_html(t)
        # decode emojis (would be removed by unidecode)
        t = self.emoji.demojize(t)
        # decode and simplify text: see unidecode library
        t = self.unidecode.unidecode(t)
        # lower case
        t = t.lower()
        # replace <PERSON> (for CC12M)
        t = replace_person_token(t)
        # remove wiki reference (for WIT)
        t = remove_wiki_ref(t)
        # remove html tags
        t = remove_html_tags(t)
        # remove urls
        t = remove_urls(t)
        # remove commas in numbers
        t = remove_comma_numbers(t)
        # handle dots in numbers and quotes - Part 1
        t = pre_process_dot_numbers(t)
        t = pre_process_quotes(t)
        t = pre_process_dates(t)
        # handle special characters
        t = handle_special_chars(t)
        # handle hashtags
        t = expand_hashtags(t, self._hashtag_processor)
        # ignore useless characters
        t = ignore_chars(t)
        # simplify quotes
        t = simplify_quotes(t)
        # all punctuation becomes commas
        t = replace_punctuation_with_commas(t)
        # handle dots in numbers and quotes - Part 2
        t = post_process_dot_numbers(t)
        t = post_process_quotes(t)
        t = post_process_dates(t)
        # handle repeating characters
        t = remove_repeating_chars(t)
        # merge quotes
        t = merge_quotes(t)
        # merge commas
        t = merge_commas(t)
        # remove multiple spaces
        t = remove_extra_spaces(t)
        # remove first and last comma
        t = remove_first_last_commas(t)
        # always start with a space
        return f" {t}"


class DalleBartTokenizer(GPTTokenizer):
    r"""
    Construct a DalleBart tokenizer based on byte-level Byte-Pair-Encoding.

    This tokenizer inherits from :class:`~paddlenlp.transformers.gpt.tokenizer.GPTTokenizer`.
    For more information regarding those methods, please refer to this superclass.

    Args:
        vocab_file (str):
            Path to the vocabulary file.
            The vocab file contains a mapping from vocabulary strings to indices.
        merges_file (str):
            Path to the merge file.
            The merge file is used to split the input sentence into "subword" units.
            The vocab file is then used to encode those units as intices.
        wiki_word_frequency_file (str):
            Path to the wiki_word_frequency file when we need normlize text.
        errors (str):
            Paradigm to follow when decoding bytes to UTF-8.
            Defaults to `'replace'`.
        max_len (int, optional):
            The maximum value of the input sequence length.
            Defaults to `None`.
        bos_token (str, optional):
            The beginning of sequence token that was used during pretraining. Can be
            used a sequence classifier token.
            Defaults to `"<s>"`.
        eos_token (str, optional):
            A special token representing the end of a sequence that was used during pretraining.
            Defaults to `"</s>"`.
        cls_token (str, optional):
            A special token used for sequence classification. It is the last token
            of the sequence when built with special tokens.
            Defaults to `"<s>"`.
        sep_token (str, optional):
            A special token separating two different sentences in the same input.
            Defaults to `"</s>"`.
        unk_token (str, optional):
            A special token representing the *unknown (out-of-vocabulary)* token.
            An unknown token is set to be `unk_token` inorder to be converted to an ID.
            Defaults to `"<unk>"`.
        pad_token (str, optional):
            A special token used to make arrays of tokens the same size for batching purposes.
            Defaults to `"<pad>"`.
        mask_token (str, optional):
            A special token representing a masked token. This is the token used
            in the masked language modeling task which the model tries to predict the original unmasked ones.
            Defaults to `"<mask>"`.

    Examples:
        .. code-block::

            from paddlenlp.transformers import DalleBartTokenizer

            tokenizer = DalleBartTokenizer.from_pretrained('dalle-mini')
            print(tokenizer('Donald Trump in Animal Crossing'))

            # {'input_ids': [0, 7083, 3252, 91, 2203, 7807, 2]}

    """
    resource_files_names = {
        "vocab_file": "vocab.json",
        "merges_file": "merges.txt",
        "wiki_word_frequency_file": "enwiki-words-frequency.txt",
    }
    pretrained_resource_files_map = {
        "vocab_file": {
            "dalle-mini": "https://bj.bcebos.com/paddlenlp/models/transformers/dallebart/dalle-mini/vocab.json",
            "dalle-mega-v16": "https://bj.bcebos.com/paddlenlp/models/transformers/dallebart/dalle-mega-v16/vocab.json",
            "dalle-mega-v26": "https://bj.bcebos.com/paddlenlp/models/transformers/dallebart/dalle-mega-v26/vocab.json",
            "dalle-mega": "https://bj.bcebos.com/paddlenlp/models/transformers/dallebart/dalle-mega-v26/vocab.json",
        },
        "merges_file": {
            "dalle-mini": "https://bj.bcebos.com/paddlenlp/models/transformers/dallebart/dalle-mini/merges.txt",
            "dalle-mega-v16": "https://bj.bcebos.com/paddlenlp/models/transformers/dallebart/dalle-mega-v16/merges.txt",
            "dalle-mega-v26": "https://bj.bcebos.com/paddlenlp/models/transformers/dallebart/dalle-mega-v26/merges.txt",
            "dalle-mega": "https://bj.bcebos.com/paddlenlp/models/transformers/dallebart/dalle-mega-v26/merges.txt",
        },
        "wiki_word_frequency_file": {
            "dalle-mini": "https://bj.bcebos.com/paddlenlp/models/transformers/dallebart/dalle-mini/enwiki-words-frequency.txt",
            "dalle-mega-v16": "https://bj.bcebos.com/paddlenlp/models/transformers/dallebart/dalle-mega-v16/enwiki-words-frequency.txt",
            "dalle-mega-v26": "https://bj.bcebos.com/paddlenlp/models/transformers/dallebart/dalle-mega-v26/enwiki-words-frequency.txt",
            "dalle-mega": "https://bj.bcebos.com/paddlenlp/models/transformers/dallebart/dalle-mega-v26/enwiki-words-frequency.txt",
        },
    }
    pretrained_init_configuration = {
        "dalle-mini": {"normalize_text": True},
        "dalle-mega-v16": {"normalize_text": True},
        "dalle-mega-v26": {"normalize_text": True},
        "dalle-mega": {"normalize_text": True},
    }
    max_model_input_sizes = PRETRAINED_POSITIONAL_EMBEDDINGS_SIZES

    def __init__(
        self,
        vocab_file,
        merges_file,
        wiki_word_frequency_file,
        normalize_text=True,
        errors="replace",
        max_len=None,
        bos_token="<s>",
        eos_token="</s>",
        cls_token="<s>",
        sep_token="</s>",
        unk_token="<unk>",
        pad_token="<pad>",
        mask_token="<mask>",
        **kwargs
    ):

        bos_token = AddedToken(bos_token, lstrip=False, rstrip=False) if isinstance(bos_token, str) else bos_token
        eos_token = AddedToken(eos_token, lstrip=False, rstrip=False) if isinstance(eos_token, str) else eos_token
        sep_token = AddedToken(sep_token, lstrip=False, rstrip=False) if isinstance(sep_token, str) else sep_token
        cls_token = AddedToken(cls_token, lstrip=False, rstrip=False) if isinstance(cls_token, str) else cls_token
        unk_token = AddedToken(unk_token, lstrip=False, rstrip=False) if isinstance(unk_token, str) else unk_token
        pad_token = AddedToken(pad_token, lstrip=False, rstrip=False) if isinstance(pad_token, str) else pad_token

        # Mask token behave like a normal word, i.e. include the space before it
        mask_token = AddedToken(mask_token, lstrip=True, rstrip=False) if isinstance(mask_token, str) else mask_token

        self._build_special_tokens_map_extended(
            bos_token=bos_token,
            eos_token=eos_token,
            sep_token=sep_token,
            cls_token=cls_token,
            unk_token=unk_token,
            pad_token=pad_token,
            mask_token=mask_token,
        )
        self.normalize_text = normalize_text
        # in order to save wiki_word_frequency_file, we need set this attr
        self._wiki_word_frequency_file = wiki_word_frequency_file
        if self.normalize_text:
            self.text_processor = TextNormalizer(wiki_word_frequency_file)
        super().__init__(vocab_file, merges_file, errors, max_len, pad_token, eos_token, unk_token)

    def _bpe_encode(self, text):
        bpe_tokens = []
        re = try_import("regex")
        for token in re.findall(self.pat, text):
            token = "".join(self.byte_encoder[b] for b in token.encode("utf-8"))
            bpe_tokens.extend(bpe_token for bpe_token in self.bpe(token).split(" "))
        return bpe_tokens

    def build_inputs_with_special_tokens(self, token_ids_0, token_ids_1=None):
        """
        Build model inputs from a sequence or a pair of sequence for sequence classification
        tasks by concatenating and adding special tokens.
        """
        _cls = [self.cls_token_id]
        _sep = [self.sep_token_id]
        if token_ids_1 is None:
            return _cls + token_ids_0 + _sep
        return _cls + token_ids_0 + _sep + _sep + token_ids_1 + _sep

    def get_special_tokens_mask(self, token_ids_0, token_ids_1=None, already_has_special_tokens=False):
        """
        Retrieves sequence ids from a token list that has no special tokens added. This method is
        called when adding special tokens using the tokenizer ``encode`` methods.
        """
        if already_has_special_tokens:
            return super().get_special_tokens_mask(
                token_ids_0=token_ids_0, token_ids_1=token_ids_1, already_has_special_tokens=True
            )
        if token_ids_1 is None:
            return [1] + ([0] * len(token_ids_0)) + [1]
        return [1] + ([0] * len(token_ids_0)) + [1, 1] + ([0] * len(token_ids_1)) + [1]

    def create_token_type_ids_from_sequences(self, token_ids_0, token_ids_1=None):
        """
        Create a mask from the two sequences passed to be used in a sequence-pair classification task.
        """
        sep = [self.sep_token_id]
        cls = [self.cls_token_id]

        if token_ids_1 is None:
            return len(cls + token_ids_0 + sep) * [0]
        return len(cls + token_ids_0 + sep + sep + token_ids_1 + sep) * [0]

    def __call__(
        self,
        text,
        text_pair=None,
        max_length=64,  # default
        stride=0,
        is_split_into_words=False,
        padding="max_length",  # default
        truncation=True,  # default
        return_position_ids=False,
        return_token_type_ids=False,  # don't return token_type_ids
        return_attention_mask=True,  # default
        return_length=False,
        return_overflowing_tokens=False,
        return_special_tokens_mask=False,
        return_dict=True,
        return_offsets_mapping=False,
        add_special_tokens=True,
        pad_to_multiple_of=None,
        return_tensors=None,
        verbose: bool = True,
        **kwargs
    ):
        if self.normalize_text:
            is_batched = isinstance(text, (list, tuple))
            if is_batched:
                text = [self.text_processor(t) for t in text]
            else:
                text = self.text_processor(text)
        return super().__call__(
            text,
            text_pair,
            max_length,
            stride,
            is_split_into_words,
            padding,
            truncation,
            return_position_ids,
            return_token_type_ids,
            return_attention_mask,
            return_length,
            return_overflowing_tokens,
            return_special_tokens_mask,
            return_dict,
            return_offsets_mapping,
            add_special_tokens,
            pad_to_multiple_of,
            return_tensors,
            verbose,
            **kwargs,
        )



# Copyright (c) 2022 PaddlePaddle Authors. All Rights Reserved.
# Copyright 2019-present, Facebook, Inc and the HuggingFace Inc. team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import itertools
import math

import paddle
import paddle.nn as nn
import paddle.nn.functional as F

from .. import PretrainedModel, register_base_model
from ..albert.modeling import ACT2FN

__all__ = [
    "XLMModel",
    "XLMPretrainedModel",
    "XLMWithLMHeadModel",
    "XLMForSequenceClassification",
    "XLMForTokenClassification",
    "XLMForQuestionAnsweringSimple",
    "XLMForMultipleChoice",
]

INF = 1e4


class SinusoidalPositionalEmbedding(nn.Embedding):
    def __init__(self, num_embeddings, embedding_dim):
        super().__init__(num_embeddings, embedding_dim)
        self.weight = self._init_weight(self.weight)

    @staticmethod
    def _init_weight(out):
        n_pos, dim = paddle.shape(out)
        out.stop_gradient = True
        position_ids = paddle.arange(0, n_pos, dtype=out.dtype).unsqueeze(1)
        indices = paddle.arange(0, dim // 2, dtype=out.dtype).unsqueeze(0)
        indices = 10000.0 ** (-2 * indices / dim)
        embeddings = paddle.matmul(position_ids, indices)
        out[:, 0::2] = paddle.sin(embeddings)
        out[:, 1::2] = paddle.cos(embeddings)
        return out

    @paddle.no_grad()
    def forward(self, position_ids):
        return super().forward(position_ids)


def get_masks(seqlen, lengths, causal, padding_mask=None):
    """
    Generate hidden states mask, and optionally an attention mask.
    """
    alen = paddle.arange(0, seqlen, dtype="int64")
    if padding_mask is not None:
        mask = padding_mask
    else:
        mask = alen < lengths[:, None]

    # attention mask is the same as mask, or triangular inferior attention (causal)
    bs = paddle.shape(lengths)[0]
    if causal:
        attn_mask = paddle.tile(alen[None, None, :], (bs, seqlen, 1)) <= alen[None, :, None]
    else:
        attn_mask = mask

    return mask, attn_mask


class MultiHeadAttention(nn.Layer):

    NEW_ID = itertools.count()

    def __init__(self, n_heads, dim, attention_probs_dropout_prob):
        super().__init__()
        self.layer_id = next(MultiHeadAttention.NEW_ID)
        self.dim = dim
        self.n_heads = n_heads
        assert self.dim % self.n_heads == 0
        self.q_lin = nn.Linear(dim, dim)
        self.k_lin = nn.Linear(dim, dim)
        self.v_lin = nn.Linear(dim, dim)
        self.out_lin = nn.Linear(dim, dim)
        self.dropout = nn.Dropout(attention_probs_dropout_prob)
        self.dim_per_head = self.dim // self.n_heads

    def shape(self, x):
        """projection"""
        return x.reshape([0, 0, self.n_heads, self.dim_per_head]).transpose([0, 2, 1, 3])

    def unshape(self, x):
        """compute context"""
        return x.transpose([0, 2, 1, 3]).reshape([0, 0, self.n_heads * self.dim_per_head])

    def forward(self, input, mask, kv=None, cache=None, output_attentions=False):
        """
        Self-attention (if kv is None) or attention over source sentence (provided by kv).
        """
        # Input is (bs, qlen, dim)
        # Mask is (bs, klen) (non-causal) or (bs, klen, klen)
        bs, qlen, dim = paddle.shape(input)
        if kv is None:
            klen = qlen if cache is None else cache["seqlen"] + qlen
        else:
            klen = paddle.shape(kv)[1]

        mask_reshape = (bs, 1, qlen, klen) if mask.ndim == 3 else (bs, 1, 1, klen)

        q = self.shape(self.q_lin(input))  # (bs, n_heads, qlen, dim_per_head)
        if kv is None:
            k = self.shape(self.k_lin(input))  # (bs, n_heads, qlen, dim_per_head)
            v = self.shape(self.v_lin(input))  # (bs, n_heads, qlen, dim_per_head)
        elif cache is None or self.layer_id not in cache:
            k = v = kv
            k = self.shape(self.k_lin(k))  # (bs, n_heads, qlen, dim_per_head)
            v = self.shape(self.v_lin(v))  # (bs, n_heads, qlen, dim_per_head)

        if cache is not None:
            if self.layer_id in cache:
                if kv is None:
                    k_, v_ = cache[self.layer_id]
                    k = paddle.concat([k_, k], axis=2)  # (bs, n_heads, klen, dim_per_head)
                    v = paddle.concat([v_, v], axis=2)  # (bs, n_heads, klen, dim_per_head)
                else:
                    k, v = cache[self.layer_id]
            cache[self.layer_id] = (k, v)

        q = q / math.sqrt(self.dim_per_head)  # (bs, n_heads, qlen, dim_per_head)

        scores = paddle.matmul(q, k, transpose_y=True)  # (bs, n_heads, qlen, klen)

        mask = mask.reshape(mask_reshape)  # (bs, n_heads, qlen, klen)

        scores = scores + (mask.astype(scores.dtype) - 1) * INF

        weights = F.softmax(scores, axis=-1)  # (bs, n_heads, qlen, klen)
        weights = self.dropout(weights)  # (bs, n_heads, qlen, klen)

        context = paddle.matmul(weights, v)  # (bs, n_heads, qlen, dim_per_head)
        context = self.unshape(context)  # (bs, qlen, dim)

        outputs = (self.out_lin(context),)
        if output_attentions:
            outputs = outputs + (weights,)
        return outputs


class TransformerFFN(nn.Layer):
    def __init__(self, in_dim, dim_hidden, out_dim, hidden_act, dropout_prob):
        super().__init__()
        self.lin1 = nn.Linear(in_dim, dim_hidden)
        self.lin2 = nn.Linear(dim_hidden, out_dim)
        self.dropout = nn.Dropout(dropout_prob)
        self.act = ACT2FN[hidden_act]

    def forward(self, x):
        x = self.lin1(x)
        x = self.act(x)
        x = self.lin2(x)
        x = self.dropout(x)
        return x


class XLMPretrainedModel(PretrainedModel):
    """
    An abstract class for pretrained XLM models. It provides XLM related
    `model_config_file`, `resource_files_names`, `pretrained_resource_files_map`,
    `pretrained_init_configuration`, `base_model_prefix` for downloading and
    loading pretrained models.
    See :class:`~paddlenlp.transformers.model_utils.PretrainedModel` for more details.
    """

    pretrained_init_configuration = {
        "xlm-mlm-en-2048": {
            "is_encoder": True,
            "causal": False,
            "n_langs": 1,
            "use_lang_embeddings": True,
            "vocab_size": 30145,
            "pad_token_id": 2,
            "hidden_size": 2048,
            "num_attention_heads": 16,
            "num_hidden_layers": 12,
            "hidden_dropout_prob": 0.1,
            "attention_probs_dropout_prob": 0.1,
            "max_position_embeddings": 512,
            "use_sinusoidal_embeddings": False,
            "layer_norm_eps": 1e-12,
            "hidden_act": "gelu",
            "embed_init_std": 0.015625,
            "init_std": 0.02,
            "lang_id": 0,
            "lang2id": None,
        },
        "xlm-mlm-ende-1024": {
            "is_encoder": True,
            "causal": False,
            "n_langs": 2,
            "use_lang_embeddings": True,
            "vocab_size": 64699,
            "pad_token_id": 2,
            "hidden_size": 1024,
            "num_attention_heads": 8,
            "num_hidden_layers": 6,
            "hidden_dropout_prob": 0.1,
            "attention_probs_dropout_prob": 0.1,
            "max_position_embeddings": 512,
            "use_sinusoidal_embeddings": False,
            "layer_norm_eps": 1e-12,
            "hidden_act": "gelu",
            "embed_init_std": 0.02209708691207961,
            "init_std": 0.02,
            "lang_id": 1,
            "lang2id": {"de": 0, "en": 1},
        },
        "xlm-mlm-enfr-1024": {
            "is_encoder": True,
            "causal": False,
            "n_langs": 2,
            "use_lang_embeddings": True,
            "vocab_size": 64139,
            "pad_token_id": 2,
            "hidden_size": 1024,
            "num_attention_heads": 8,
            "num_hidden_layers": 6,
            "hidden_dropout_prob": 0.1,
            "attention_probs_dropout_prob": 0.1,
            "max_position_embeddings": 512,
            "use_sinusoidal_embeddings": False,
            "layer_norm_eps": 1e-12,
            "hidden_act": "gelu",
            "embed_init_std": 0.02209708691207961,
            "init_std": 0.02,
            "lang_id": 0,
            "lang2id": {"en": 0, "fr": 1},
        },
        "xlm-mlm-enro-1024": {
            "is_encoder": True,
            "causal": False,
            "n_langs": 2,
            "use_lang_embeddings": True,
            "vocab_size": 64592,
            "pad_token_id": 2,
            "hidden_size": 1024,
            "num_attention_heads": 8,
            "num_hidden_layers": 6,
            "hidden_dropout_prob": 0.1,
            "attention_probs_dropout_prob": 0.1,
            "max_position_embeddings": 512,
            "use_sinusoidal_embeddings": False,
            "layer_norm_eps": 1e-12,
            "hidden_act": "gelu",
            "embed_init_std": 0.02209708691207961,
            "init_std": 0.02,
            "lang_id": 0,
            "lang2id": {"en": 0, "ro": 1},
        },
        "xlm-mlm-tlm-xnli15-1024": {
            "is_encoder": True,
            "causal": False,
            "n_langs": 15,
            "use_lang_embeddings": True,
            "vocab_size": 95000,
            "pad_token_id": 2,
            "hidden_size": 1024,
            "num_attention_heads": 8,
            "num_hidden_layers": 12,
            "hidden_dropout_prob": 0.1,
            "attention_probs_dropout_prob": 0.1,
            "max_position_embeddings": 512,
            "use_sinusoidal_embeddings": False,
            "layer_norm_eps": 1e-12,
            "hidden_act": "gelu",
            "embed_init_std": 0.02209708691207961,
            "init_std": 0.02,
            "lang_id": 4,
            "lang2id": {
                "ar": 0,
                "bg": 1,
                "de": 2,
                "el": 3,
                "en": 4,
                "es": 5,
                "fr": 6,
                "hi": 7,
                "ru": 8,
                "sw": 9,
                "th": 10,
                "tr": 11,
                "ur": 12,
                "vi": 13,
                "zh": 14,
            },
        },
        "xlm-mlm-xnli15-1024": {
            "is_encoder": True,
            "causal": False,
            "n_langs": 15,
            "use_lang_embeddings": True,
            "vocab_size": 95000,
            "pad_token_id": 2,
            "hidden_size": 1024,
            "num_attention_heads": 8,
            "num_hidden_layers": 12,
            "hidden_dropout_prob": 0.1,
            "attention_probs_dropout_prob": 0.1,
            "max_position_embeddings": 512,
            "use_sinusoidal_embeddings": False,
            "layer_norm_eps": 1e-12,
            "hidden_act": "gelu",
            "embed_init_std": 0.02209708691207961,
            "init_std": 0.02,
            "lang_id": 4,
            "lang2id": {
                "ar": 0,
                "bg": 1,
                "de": 2,
                "el": 3,
                "en": 4,
                "es": 5,
                "fr": 6,
                "hi": 7,
                "ru": 8,
                "sw": 9,
                "th": 10,
                "tr": 11,
                "ur": 12,
                "vi": 13,
                "zh": 14,
            },
        },
        "xlm-clm-enfr-1024": {
            "is_encoder": True,
            "causal": False,
            "n_langs": 2,
            "use_lang_embeddings": True,
            "vocab_size": 64139,
            "pad_token_id": 2,
            "hidden_size": 1024,
            "num_attention_heads": 8,
            "num_hidden_layers": 6,
            "hidden_dropout_prob": 0.1,
            "attention_probs_dropout_prob": 0.1,
            "max_position_embeddings": 512,
            "use_sinusoidal_embeddings": False,
            "layer_norm_eps": 1e-12,
            "hidden_act": "gelu",
            "embed_init_std": 0.02209708691207961,
            "init_std": 0.02,
            "lang_id": 0,
            "lang2id": {"en": 0, "fr": 1},
        },
        "xlm-clm-ende-1024": {
            "is_encoder": True,
            "causal": False,
            "n_langs": 2,
            "use_lang_embeddings": True,
            "vocab_size": 64699,
            "pad_token_id": 2,
            "hidden_size": 1024,
            "num_attention_heads": 8,
            "num_hidden_layers": 6,
            "hidden_dropout_prob": 0.1,
            "attention_probs_dropout_prob": 0.1,
            "max_position_embeddings": 512,
            "use_sinusoidal_embeddings": False,
            "layer_norm_eps": 1e-12,
            "hidden_act": "gelu",
            "embed_init_std": 0.02209708691207961,
            "init_std": 0.02,
            "lang_id": 1,
            "lang2id": {"de": 0, "en": 1},
        },
        "xlm-mlm-17-1280": {
            "is_encoder": True,
            "causal": False,
            "n_langs": 17,
            "use_lang_embeddings": False,
            "vocab_size": 200000,
            "pad_token_id": 2,
            "hidden_size": 1280,
            "num_attention_heads": 16,
            "num_hidden_layers": 16,
            "hidden_dropout_prob": 0.1,
            "attention_probs_dropout_prob": 0.1,
            "max_position_embeddings": 512,
            "use_sinusoidal_embeddings": False,
            "layer_norm_eps": 1e-12,
            "hidden_act": "gelu",
            "embed_init_std": 0.01976423537605237,
            "init_std": 0.02,
            "lang_id": 2,
            "lang2id": {
                "ar": 0,
                "de": 1,
                "en": 2,
                "es": 3,
                "fr": 4,
                "hi": 5,
                "it": 6,
                "ja": 7,
                "ko": 8,
                "nl": 9,
                "pl": 10,
                "pt": 11,
                "ru": 12,
                "sv": 13,
                "tr": 14,
                "vi": 15,
                "zh": 16,
            },
        },
        "xlm-mlm-100-1280": {
            "is_encoder": True,
            "causal": False,
            "n_langs": 100,
            "use_lang_embeddings": False,
            "vocab_size": 200000,
            "pad_token_id": 2,
            "hidden_size": 1280,
            "num_attention_heads": 16,
            "num_hidden_layers": 16,
            "hidden_dropout_prob": 0.1,
            "attention_probs_dropout_prob": 0.1,
            "max_position_embeddings": 512,
            "use_sinusoidal_embeddings": False,
            "layer_norm_eps": 1e-12,
            "hidden_act": "gelu",
            "embed_init_std": 0.01976423537605237,
            "init_std": 0.02,
            "lang_id": 23,
            "lang2id": {
                "af": 0,
                "als": 1,
                "am": 2,
                "an": 3,
                "ang": 4,
                "ar": 5,
                "arz": 6,
                "ast": 7,
                "az": 8,
                "bar": 9,
                "be": 10,
                "bg": 11,
                "bn": 12,
                "br": 13,
                "bs": 14,
                "ca": 15,
                "ceb": 16,
                "ckb": 17,
                "cs": 18,
                "cy": 19,
                "da": 20,
                "de": 21,
                "el": 22,
                "en": 23,
                "eo": 24,
                "es": 25,
                "et": 26,
                "eu": 27,
                "fa": 28,
                "fi": 29,
                "fr": 30,
                "fy": 31,
                "ga": 32,
                "gan": 33,
                "gl": 34,
                "gu": 35,
                "he": 36,
                "hi": 37,
                "hr": 38,
                "hu": 39,
                "hy": 40,
                "ia": 41,
                "id": 42,
                "is": 43,
                "it": 44,
                "ja": 45,
                "jv": 46,
                "ka": 47,
                "kk": 48,
                "kn": 49,
                "ko": 50,
                "ku": 51,
                "la": 52,
                "lb": 53,
                "lt": 54,
                "lv": 55,
                "mk": 56,
                "ml": 57,
                "mn": 58,
                "mr": 59,
                "ms": 60,
                "my": 61,
                "nds": 62,
                "ne": 63,
                "nl": 64,
                "nn": 65,
                "no": 66,
                "oc": 67,
                "pl": 68,
                "pt": 69,
                "ro": 70,
                "ru": 71,
                "scn": 72,
                "sco": 73,
                "sh": 74,
                "si": 75,
                "simple": 76,
                "sk": 77,
                "sl": 78,
                "sq": 79,
                "sr": 80,
                "sv": 81,
                "sw": 82,
                "ta": 83,
                "te": 84,
                "th": 85,
                "tl": 86,
                "tr": 87,
                "tt": 88,
                "uk": 89,
                "ur": 90,
                "uz": 91,
                "vi": 92,
                "war": 93,
                "wuu": 94,
                "yi": 95,
                "zh": 96,
                "zh_classical": 97,
                "zh_min_nan": 98,
                "zh_yue": 99,
            },
        },
    }

    pretrained_resource_files_map = {
        "model_state": {
            "xlm-mlm-en-2048": "https://bj.bcebos.com/paddlenlp/models/transformers/xlm/xlm-mlm-en-2048/model_state.pdparams",
            "xlm-mlm-ende-1024": "https://bj.bcebos.com/paddlenlp/models/transformers/xlm/xlm-mlm-ende-1024/model_state.pdparams",
            "xlm-mlm-enfr-1024": "https://bj.bcebos.com/paddlenlp/models/transformers/xlm/xlm-mlm-enfr-1024/model_state.pdparams",
            "xlm-mlm-enro-1024": "https://bj.bcebos.com/paddlenlp/models/transformers/xlm/xlm-mlm-enro-1024/model_state.pdparams",
            "xlm-mlm-tlm-xnli15-1024": "https://bj.bcebos.com/paddlenlp/models/transformers/xlm/xlm-mlm-tlm-xnli15-1024/model_state.pdparams",
            "xlm-mlm-xnli15-1024": "https://bj.bcebos.com/paddlenlp/models/transformers/xlm/xlm-mlm-xnli15-1024/model_state.pdparams",
            "xlm-clm-enfr-1024": "https://bj.bcebos.com/paddlenlp/models/transformers/xlm/xlm-clm-enfr-1024/model_state.pdparams",
            "xlm-clm-ende-1024": "https://bj.bcebos.com/paddlenlp/models/transformers/xlm/xlm-clm-ende-1024/model_state.pdparams",
            "xlm-mlm-17-1280": "https://bj.bcebos.com/paddlenlp/models/transformers/xlm/xlm-mlm-17-1280/model_state.pdparams",
            "xlm-mlm-100-1280": "https://bj.bcebos.com/paddlenlp/models/transformers/xlm/xlm-mlm-100-1280/model_state.pdparams",
        }
    }
    base_model_prefix = "xlm"

    def init_weights(self):
        # Initialize weights
        self.apply(self._init_weights)

    def _init_weights(self, layer):
        """Initialization hook"""
        if isinstance(layer, nn.Embedding):
            new_weight = paddle.normal(
                mean=0.0,
                std=self.embed_init_std if hasattr(self, "embed_init_std") else self.xlm.config["embed_init_std"],
                shape=layer.weight.shape,
            )
            if layer._padding_idx is not None:
                new_weight[layer._padding_idx] = paddle.zeros_like(new_weight[layer._padding_idx])
            layer.weight.set_value(new_weight)
        elif isinstance(layer, nn.Linear):
            layer.weight.set_value(
                paddle.normal(
                    mean=0.0,
                    std=self.init_std if hasattr(self, "init_std") else self.xlm.config["init_std"],
                    shape=layer.weight.shape,
                )
            )
            if layer.bias is not None:
                layer.bias.set_value(paddle.zeros_like(layer.bias))
        elif isinstance(layer, nn.LayerNorm):
            layer.bias.set_value(paddle.zeros_like(layer.bias))
            layer.weight.set_value(paddle.full_like(layer.weight, 1.0))


@register_base_model
class XLMModel(XLMPretrainedModel):
    """
    The bare XLM Model transformer outputting raw hidden-states.

    This model inherits from :class:`~paddlenlp.transformers.model_utils.PretrainedModel`.
    Refer to the superclass documentation for the generic methods.

    This model is also a Paddle `paddle.nn.Layer <https://www.paddlepaddle.org.cn/documentation
    /docs/en/api/paddle/fluid/dygraph/layers/Layer_en.html>`__ subclass. Use it as a regular Paddle Layer
    and refer to the Paddle documentation for all matter related to general usage and behavior.

    Args:
        vocab_size (int, optional):
            Vocabulary size of `inputs_ids` in `XLMModel`. Also is the vocab size of token embedding matrix.
            Defines the number of different tokens that can be represented by the `inputs_ids` passed when calling `XLMModel`.
            Defaults to `95000`.
        is_encoder(bool, optional):
            Whether or not the initialized model should be a transformer encoder or decoder as seen in Vaswani et al.
            Defaults to `True`.
        causal (bool, optional):
            Whether or not the model should behave in a causal manner. Causal models use a triangular attention mask in
            order to only attend to the left-side context instead if a bidirectional context.
            Defaults to `False`.
        n_langs (int, optional):
            The number of languages the model handles. Set to 1 for monolingual models.
            Defaults to `15`.
        use_lang_embeddings (bool, optional)
            Whether to use language embeddings. Some models use additional language embeddings.
            Defaults to `True`.
        hidden_size (int, optional):
            Dimensionality of the embedding layer, encoder layer. Defaults to `1024`.
        hidden_act (str, optional):
            The non-linear activation function in the feed-forward layer.
            ``"gelu"``, ``"relu"`` and any other paddle supported activation functions
            are supported. Defaults to `"gelu"`.
        num_attention_heads (int, optional):
            Number of attention heads for each attention layer in the Transformer encoder.
            Defaults to `8`.
        num_hidden_layers (int, optional):
            Number of hidden layers in the Transformer encoder. Defaults to `12`.
        hidden_dropout_prob (float, optional):
            The dropout probability for all fully connected layers in the embeddings and encoder.
            Defaults to `0.1`.
        attention_probs_dropout_prob (float, optional):
            The dropout probability used in MultiHeadAttention in all encoder layers to drop some attention target.
            Defaults to `0.1`.
        max_position_embeddings (int, optional):
            The maximum value of the dimensionality of position encoding, which dictates the maximum supported length of an input
            sequence. Defaults to `512`.
        use_sinusoidal_embeddings (bool, optional):
            Whether or not to use sinusoidal positional embeddings instead of absolute positional embeddings.
            Defaults to `False`.
        layer_norm_eps (float, optional):
            The epsilon used by the layer normalization layers.
            Defaults to 1e-12.
        embed_init_std (float, optional):
            The standard deviation of the truncated_normal_initializer for initializing the embedding matrices.
            Defaults to `(1024*2)^-0.5`.
        init_std (int, optional):
            The standard deviation of the truncated_normal_initializer for initializing all weight matrices except the
            embedding matrices.
            Defaults to `0.02`.
        pad_token_id (int, optional):
            The index of padding token in the token vocabulary.
            Defaults to `2`.
        lang_id (int, optional):
            The ID of the language used by the model. This parameter is used when generating text in a given language.
            Defaults to 4.
        lang2id (Dict[str, int], optional):
            Dictionary mapping languages string identifiers to their IDs.
    """

    def __init__(
        self,
        vocab_size=95000,
        is_encoder=True,
        causal=False,
        n_langs=15,
        use_lang_embeddings=True,
        hidden_size=1024,
        hidden_act="gelu",
        num_attention_heads=8,
        num_hidden_layers=12,
        hidden_dropout_prob=0.1,
        attention_probs_dropout_prob=0.1,
        max_position_embeddings=512,
        use_sinusoidal_embeddings=False,
        layer_norm_eps=1e-12,
        embed_init_std=2048**-0.5,
        init_std=0.02,
        pad_token_id=2,
        lang_id=4,
        lang2id={
            "ar": 0,
            "bg": 1,
            "de": 2,
            "el": 3,
            "en": 4,
            "es": 5,
            "fr": 6,
            "hi": 7,
            "ru": 8,
            "sw": 9,
            "th": 10,
            "tr": 11,
            "ur": 12,
            "vi": 13,
            "zh": 14,
        },
    ):
        super().__init__()
        self.causal = causal
        self.num_hidden_layers = num_hidden_layers
        self.pad_token_id = pad_token_id
        self.hidden_size = hidden_size
        self.embed_init_std = embed_init_std
        self.init_std = init_std
        self.use_lang_embeddings = use_lang_embeddings
        self.n_langs = n_langs
        if not is_encoder:
            raise NotImplementedError("Currently XLM can only be used as an encoder")
        assert (
            hidden_size % num_attention_heads == 0
        ), "xlm model's hidden_size must be a multiple of num_attention_heads"

        # embeddings
        if use_sinusoidal_embeddings:
            self.position_embeddings = SinusoidalPositionalEmbedding(max_position_embeddings, hidden_size)
        else:
            self.position_embeddings = nn.Embedding(max_position_embeddings, hidden_size)
        if n_langs > 1 and use_lang_embeddings:
            self.lang_embeddings = nn.Embedding(n_langs, hidden_size)
        self.embeddings = nn.Embedding(vocab_size, hidden_size)
        self.layer_norm_emb = nn.LayerNorm(hidden_size, epsilon=layer_norm_eps)

        self.attentions = nn.LayerList()
        self.layer_norm1 = nn.LayerList()
        self.ffns = nn.LayerList()
        self.layer_norm2 = nn.LayerList()
        self.dropout = nn.Dropout(hidden_dropout_prob)

        for _ in range(self.num_hidden_layers):
            self.attentions.append(MultiHeadAttention(num_attention_heads, hidden_size, attention_probs_dropout_prob))
            self.layer_norm1.append(nn.LayerNorm(hidden_size, epsilon=layer_norm_eps))

            self.ffns.append(
                TransformerFFN(
                    hidden_size,
                    hidden_size * 4,
                    hidden_size,
                    hidden_act,
                    hidden_dropout_prob,
                )
            )
            self.layer_norm2.append(nn.LayerNorm(hidden_size, epsilon=layer_norm_eps))

        self.register_buffer(
            "position_ids",
            paddle.arange(0, max_position_embeddings).reshape((1, -1)),
            persistable=False,
        )
        self.init_weights()

    def forward(
        self,
        input_ids=None,
        langs=None,
        attention_mask=None,
        position_ids=None,
        lengths=None,
        cache=None,
        output_attentions=False,
        output_hidden_states=False,
    ):
        r"""
        The XLMModel forward method, overrides the `__call__()` special method.

        Args:
            input_ids (Tensor):
                Indices of input sequence tokens in the vocabulary. They are
                numerical representations of tokens that build the input sequence.
                Its data type should be `int64` and it has a shape of [batch_size, sequence_length].
            langs (Tensor, optional):
                A parallel sequence of tokens to be used to indicate the language of each token in the input. Indices are
                languages ids which can be obtained from the language names by using two conversion mappings provided in
                the configuration of the model (only provided for multilingual models). More precisely, the *language name
                to language id* mapping is in `model.config['lang2id']` (which is a dictionary string to int).
                Shape as [batch_size, sequence_length] and dtype as int64. Defaults to `None`.
            attention_mask (Tensor, optional):
                Mask used in multi-head attention to avoid performing attention on to some
                unwanted positions, usually the paddings or the subsequent positions.
                Its data type can be int, float and bool.
                When the data type is bool, the `masked` tokens have `False` values and the others
                have `True` values.
                When the data type is int, the `masked` tokens have `0` values and the others have `1` values.
                When the data type is float, the `masked` tokens have `0.0` values and the others have `1.0` values.
                It is a tensor with shape broadcasted to `[batch_size, num_attention_heads, sequence_length, sequence_length]`.
                Defaults to `None`, which means nothing needed to be prevented attention to.
            position_ids (Tensor, optional):
                Indices of positions of each input sequence tokens in the position embeddings. Selected
                in the range `[0, max_position_embeddings - 1]`.
                Shape as [batch_size, sequence_length] and dtype as int64. Defaults to `None`.
            lengths (Tensor, optional):
                Length of each sentence that can be used to avoid performing attention on padding token indices. You can
                also use *attention_mask* for the same result (see above), kept here for compatibility. Indices selected in
                `[0, ..., sequence_length]`.
                Shape as [batch_size] and dtype as int64. Defaults to `None`.
            cache (Tuple[Tuple[Tensor]], optional):
                Contains pre-computed hidden-states (key and values in the attention blocks)
                as computed by the model. Can be used to speed up sequential decoding.
                The `input_ids` which have their past given to this model should not be
                passed as input ids as they have already been computed.
                Defaults to `None`.
            output_attentions (bool, optional):
                Whether or not to return the attentions tensors of all attention layers.
                Defaults to `False`.
            output_hidden_states (bool, optional):
                Whether or not to return the output of all hidden layers.
                Defaults to `False`.

        Returns:
            tuple: Returns tuple (`last_hidden_state`, `hidden_states`, `attentions`)

            With the fields:

            - `last_hidden_state` (Tensor):
                Sequence of hidden-states at the last layer of the model.
                It's data type should be float32 and its shape is [batch_size, sequence_length, hidden_size].

            - `hidden_states` (tuple(Tensor), optional):
                returned when `output_hidden_states=True` is passed.
                Tuple of `Tensor` (one for the output of the embeddings + one for the output of
                each layer). Each Tensor has a data type of float32 and its shape is
                [batch_size, sequence_length, hidden_size].

            - `attentions` (tuple(Tensor), optional):
                returned when `output_attentions=True` is passed.
                Tuple of `Tensor` (one for each layer) of shape. Each Tensor has a data type of
                float32 and its shape is [batch_size, num_heads, sequence_length, sequence_length].

        Example:
            .. code-block::

                import paddle
                from paddlenlp.transformers import XLMModel, XLMTokenizer

                tokenizer = XLMTokenizer.from_pretrained("xlm-mlm-tlm-xnli15-1024")
                model = XLMModel.from_pretrained("xlm-mlm-tlm-xnli15-1024")

                inputs = tokenizer("Welcome to use PaddlePaddle and PaddleNLP!", lang="en")
                inputs = {k:paddle.to_tensor([v], dtype="int64") for (k, v) in inputs.items()}
                inputs["langs"] = paddle.ones_like(inputs["input_ids"]) * tokenizer.lang2id["en"]

                last_hidden_state = model(**inputs)[0]

        """
        bs, seqlen = paddle.shape(input_ids)

        if lengths is None:
            if input_ids is not None:
                lengths = (input_ids != self.pad_token_id).sum(axis=1).astype("int64")
            else:
                lengths = paddle.to_tensor([seqlen] * bs, dtype="int64")

        # generate masks
        mask, attn_mask = get_masks(seqlen, lengths, self.causal, padding_mask=attention_mask)

        # position_ids
        if position_ids is None:
            position_ids = self.position_ids[:, :seqlen]

        # do not recompute cached elements
        if cache is not None and input_ids is not None:
            _seqlen = seqlen - cache["seqlen"]
            input_ids = input_ids[:, -_seqlen:]
            position_ids = position_ids[:, -_seqlen:]
            if langs is not None:
                langs = langs[:, -_seqlen:]
            mask = mask[:, -_seqlen:]
            attn_mask = attn_mask[:, -_seqlen:]

        # embeddings
        tensor = self.embeddings(input_ids) + self.position_embeddings(position_ids)
        if langs is not None and self.use_lang_embeddings and self.n_langs > 1:
            tensor = tensor + self.lang_embeddings(langs)

        tensor = self.layer_norm_emb(tensor)
        tensor = self.dropout(tensor)
        tensor = tensor * mask.unsqueeze(-1).astype(tensor.dtype)

        # transformer layers
        hidden_states = () if output_hidden_states else None
        attentions = () if output_attentions else None
        for i in range(self.num_hidden_layers):
            if output_hidden_states:
                hidden_states = hidden_states + (tensor,)
            # self attention
            attn_outputs = self.attentions[i](
                tensor,
                attn_mask,
                cache=cache,
                output_attentions=output_attentions,
            )
            attn = attn_outputs[0]
            if output_attentions:
                attentions = attentions + (attn_outputs[1],)
            attn = self.dropout(attn)
            tensor = tensor + attn
            tensor = self.layer_norm1[i](tensor)
            # FFN
            tensor = tensor + self.ffns[i](tensor)
            tensor = self.layer_norm2[i](tensor)
            tensor = tensor * mask.unsqueeze(-1).astype(tensor.dtype)

        # Add last hidden state
        if output_hidden_states:
            hidden_states = hidden_states + (tensor,)

        # update cache length
        if cache is not None:
            cache["seqlen"] += paddle.shape(tensor)[1]

        return tuple(v for v in [tensor, hidden_states, attentions] if v is not None)


class XLMPredLayer(nn.Layer):
    """
    Prediction layer with cross_entropy.
    """

    def __init__(
        self,
        vocab_size,
        hidden_size,
        embedding_weights=None,
    ):
        super().__init__()
        self.vocab_size = vocab_size
        if embedding_weights is None:
            self.proj = nn.Linear(hidden_size, vocab_size)
        else:
            self.bias = self.create_parameter(shape=[vocab_size], is_bias=True)
            self.proj = lambda x: paddle.matmul(x, embedding_weights, transpose_y=True) + self.bias

    def forward(self, x, y=None):
        """Compute the loss, and optionally the scores."""
        outputs = ()
        scores = self.proj(x)
        outputs = (scores,) + outputs
        if y is not None:
            loss = F.cross_entropy(scores.reshape([-1, self.vocab_size]), y.flatten(), reduction="mean")
            outputs = (loss,) + outputs
        return outputs


class XLMWithLMHeadModel(XLMPretrainedModel):
    """
    The XLM Model transformer with a masked language modeling head on top (linear
    layer with weights tied to the input embeddings).

    Args:
        xlm (:class:`XLMModel`):
            An instance of :class:`XLMModel`.

    """

    def __init__(self, xlm):
        super().__init__()
        self.xlm = xlm
        self.pred_layer = XLMPredLayer(
            xlm.config["vocab_size"],
            xlm.config["hidden_size"],
            embedding_weights=self.xlm.embeddings.weight,
        )
        self.init_weights()

    def forward(
        self, input_ids=None, langs=None, attention_mask=None, position_ids=None, lengths=None, cache=None, labels=None
    ):
        r"""
        The XLMWithLMHeadModel forward method, overrides the __call__() special method.

        Args:
            input_ids (Tensor):
                See :class:`XLMModel`.
            langs (Tensor, optional):
                See :class:`XLMModel`.
            attention_mask (Tensor, optional):
                See :class:`XLMModel`.
            position_ids (Tensor, optional):
                See :class:`XLMModel`.
            lengths (Tensor, optional):
                See :class:`XLMModel`.
            cache (Dict[str, Tensor], optional):
                See :class:`XLMModel`.
            labels (Tensor, optional):
                The Labels for computing the masked language modeling loss. Indices are selected in
                `[-100, 0, ..., vocab_size-1]` All labels set to `-100` are ignored (masked), the loss is
                only computed for labels in `[0, ..., vocab_size-1]`
                Shape as [batch_size, sequence_length] and dtype as int64. Defaults to `None`.

        Returns:
            tuple: Returns tuple `(loss, logits)`.
            With the fields:

            - `loss` (Tensor):
                returned when `labels` is provided.
                Language modeling loss (for next-token prediction).
                It's data type should be float32 and its shape is [1,].

            - `logits` (Tensor):
                Prediction scores of the language modeling head (scores for each vocabulary
                token before SoftMax).
                It's data type should be float32 and
                its shape is [batch_size, sequence_length, vocab_size].

        Example:
            .. code-block::

                import paddle
                from paddlenlp.transformers import XLMWithLMHeadModel, XLMTokenizer

                tokenizer = XLMTokenizer.from_pretrained('xlm-mlm-tlm-xnli15-1024')
                model = XLMWithLMHeadModel.from_pretrained('xlm-mlm-tlm-xnli15-1024')

                inputs = tokenizer("Welcome to use PaddlePaddle and PaddleNLP!", lang="en")
                inputs = {k:paddle.to_tensor([v], dtype="int64") for (k, v) in inputs.items()}
                inputs["langs"] = paddle.ones_like(inputs["input_ids"]) * tokenizer.lang2id["en"]
                inputs["labels"] = inputs["input_ids"]

                loss, logits = model(**inputs)


        """
        xlm_outputs = self.xlm(
            input_ids,
            langs=langs,
            attention_mask=attention_mask,
            position_ids=position_ids,
            lengths=lengths,
            cache=cache,
        )

        output = xlm_outputs[0]
        outputs = self.pred_layer(output, labels)
        return outputs + xlm_outputs[1:]


class XLMForSequenceClassification(XLMPretrainedModel):
    """
    The XLMModel with a sequence classification head on top (linear layer).
    `XLMForSequenceClassification` uses the first token in order to do the classification.

    Args:
        xlm (:class:`XLMModel`):
            An instance of :class:`XLMModel`.
        num_classes (int, optional):
            The number of classes. Defaults to `2`.
        dropout (float, optional):
            The dropout probability for output of XLM.
            If None, use the same value as `hidden_dropout_prob` of `XLMModel`
            instance `xlm`. Defaults to None.

    """

    def __init__(self, xlm, num_classes=2, dropout=None):
        super().__init__()
        self.num_classes = num_classes
        self.xlm = xlm
        dropout_prob = dropout if dropout is not None else self.xlm.config["hidden_dropout_prob"]
        self.dropout = nn.Dropout(dropout_prob)
        self.classifier = nn.Linear(self.xlm.config["hidden_size"], num_classes)
        self.init_weights()

    def forward(self, input_ids=None, langs=None, attention_mask=None, position_ids=None, lengths=None):
        r"""
        The XLMForSequenceClassification forward method, overrides the __call__() special method.

        Args:
            input_ids (Tensor):
                See :class:`XLMModel`.
            langs (Tensor, optional):
                See :class:`XLMModel`.
            attention_mask (Tensor, optional):
                See :class:`XLMModel`.
            position_ids (Tensor, optional):
                See :class:`XLMModel`.
            lengths (Tensor, optional):
                See :class:`XLMModel`.

        Returns:
            logits (Tensor):
                A tensor of the input text classification logits.
                Shape as `[batch_size, num_classes]` and dtype as float32.

        Example:
            .. code-block::

                import paddle
                from paddlenlp.transformers import XLMForSequenceClassification, XLMTokenizer

                tokenizer = XLMTokenizer.from_pretrained("xlm-mlm-tlm-xnli15-1024")
                model = XLMForSequenceClassification.from_pretrained("xlm-mlm-tlm-xnli15-1024", num_classes=2)

                inputs = tokenizer("Welcome to use PaddlePaddle and PaddleNLP!", lang="en")
                inputs = {k:paddle.to_tensor([v], dtype="int64") for (k, v) in inputs.items()}
                inputs["langs"] = paddle.ones_like(inputs["input_ids"]) * tokenizer.lang2id["en"]

                logits = model(**inputs)

        """

        sequence_output = self.xlm(
            input_ids, langs=langs, attention_mask=attention_mask, position_ids=position_ids, lengths=lengths
        )[0]
        sequence_output = self.dropout(sequence_output)
        pooled_output = sequence_output[:, 0]
        logits = self.classifier(pooled_output)

        return logits


class XLMForTokenClassification(XLMPretrainedModel):
    """
    XLMModel with a linear layer on top of the hidden-states output layer,
    designed for token classification tasks like NER tasks.

    Args:
        xlm (:class:`XLMModel`):
            An instance of :class:`XLMModel`.
        num_classes (int, optional):
            The number of classes. Defaults to `2`.
        dropout (float, optional):
            The dropout probability for output of XLM.
            If None, use the same value as `hidden_dropout_prob` of `XLMModel`
            instance `xlm`. Defaults to None.
    """

    def __init__(self, xlm, num_classes=2, dropout=None):
        super(XLMForTokenClassification, self).__init__()
        self.num_classes = num_classes
        self.xlm = xlm  # allow xlm to be config
        self.dropout = nn.Dropout(dropout if dropout is not None else self.xlm.config["hidden_dropout_prob"])
        self.classifier = nn.Linear(self.xlm.config["hidden_size"], num_classes)
        self.init_weights()

    def forward(self, input_ids=None, langs=None, attention_mask=None, position_ids=None, lengths=None):
        r"""
        The XLMForTokenClassification forward method, overrides the __call__() special method.

        Args:
            input_ids (Tensor):
                See :class:`XLMModel`.
            langs (Tensor, optional):
                See :class:`XLMModel`.
            attention_mask (Tensor, optional):
                See :class:`XLMModel`.
            position_ids (Tensor, optional):
                See :class:`XLMModel`.
            lengths (Tensor, optional):
                See :class:`XLMModel`.

        Returns:
            logits (Tensor):
                A tensor of the input token classification logits.
                Shape as `[batch_size, sequence_length, num_classes]` and dtype as `float32`.

        Example:
            .. code-block::

                import paddle
                from paddlenlp.transformers import XLMForTokenClassification, XLMTokenizer

                tokenizer = XLMTokenizer.from_pretrained("xlm-mlm-tlm-xnli15-1024")
                model = XLMForTokenClassification.from_pretrained("xlm-mlm-tlm-xnli15-1024", num_classes=2)

                inputs = tokenizer("Welcome to use PaddlePaddle and PaddleNLP!", lang="en")
                inputs = {k:paddle.to_tensor([v], dtype="int64") for (k, v) in inputs.items()}
                inputs["langs"] = paddle.ones_like(inputs["input_ids"]) * tokenizer.lang2id["en"]

                logits = model(**inputs)

        """

        sequence_output = self.xlm(
            input_ids, langs=langs, attention_mask=attention_mask, position_ids=position_ids, lengths=lengths
        )[0]
        sequence_output = self.dropout(sequence_output)
        logits = self.classifier(sequence_output)

        return logits


class XLMForQuestionAnsweringSimple(XLMPretrainedModel):
    """
    XLMModel with a span classification head on top for extractive question-answering tasks like SQuAD (a linear
    layers on top of the hidden-states output to compute `span start logits` and `span end logits`).

    Args:
        xlm (:class:`XLMModel`):
            An instance of XLMModel.
    """

    def __init__(self, xlm):
        super(XLMForQuestionAnsweringSimple, self).__init__()
        self.xlm = xlm  # allow xlm to be config
        self.classifier = nn.Linear(self.xlm.config["hidden_size"], 2)
        self.init_weights()

    def forward(self, input_ids=None, langs=None, attention_mask=None, position_ids=None, lengths=None):
        r"""
        The XLMForQuestionAnswering forward method, overrides the __call__() special method.

        Args:
            input_ids (Tensor):
                See :class:`XLMModel`.
            langs (Tensor, optional):
                See :class:`XLMModel`.
            attention_mask (Tensor, optional):
                See :class:`XLMModel`.
            position_ids (Tensor, optional):
                See :class:`XLMModel`.
            lengths (Tensor, optional):
                See :class:`XLMModel`.

        Returns:
            tuple: Returns tuple (`start_logits`, `end_logits`).

            With the fields:

            - `start_logits` (Tensor):
                A tensor of the input token classification logits, indicates the start position of the labelled span.
                Its data type should be float32 and its shape is [batch_size, sequence_length].

            - `end_logits` (Tensor):
                A tensor of the input token classification logits, indicates the end position of the labelled span.
                Its data type should be float32 and its shape is [batch_size, sequence_length].

        Example:
            .. code-block::

                import paddle
                from paddlenlp.transformers import XLMForQuestionAnswering, XLMTokenizer

                tokenizer = XLMTokenizer.from_pretrained("xlm-mlm-tlm-xnli15-1024")
                model = XLMForQuestionAnswering.from_pretrained("xlm-mlm-tlm-xnli15-1024", num_classes=2)

                inputs = tokenizer("Welcome to use PaddlePaddle and PaddleNLP!", lang="en")
                inputs = {k:paddle.to_tensor([v], dtype="int64") for (k, v) in inputs.items()}
                inputs["langs"] = paddle.ones_like(inputs["input_ids"]) * tokenizer.lang2id["en"]

                outputs = model(**inputs)

                start_logits = outputs[0]
                end_logits = outputs[1]

        """

        sequence_output = self.xlm(
            input_ids, langs=langs, attention_mask=attention_mask, position_ids=position_ids, lengths=lengths
        )[0]
        logits = self.classifier(sequence_output)
        start_logits, end_logits = paddle.unstack(x=logits, axis=-1)

        return start_logits, end_logits


class XLMForMultipleChoice(XLMPretrainedModel):
    """
    XLMModel with a linear layer on top of the hidden-states output layer,
    designed for multiple choice tasks like RocStories/SWAG tasks.

    Args:
        xlm (:class:`XLMModel`):
            An instance of XLMModel.
        num_choices (int, optional):
            The number of choices. Defaults to `2`.
        dropout (float, optional):
            The dropout probability for output of XLM.
            If None, use the same value as `hidden_dropout_prob` of `XLMModel`
            instance `xlm`. Defaults to None.
    """

    def __init__(self, xlm, num_choices=2, dropout=None):
        super(XLMForMultipleChoice, self).__init__()
        self.num_choices = num_choices
        self.xlm = xlm
        self.dropout = nn.Dropout(dropout if dropout is not None else self.xlm.config["hidden_dropout_prob"])
        self.classifier = nn.Linear(self.xlm.config["hidden_size"], 1)
        self.init_weights()

    def forward(self, input_ids=None, langs=None, attention_mask=None, position_ids=None, lengths=None):
        r"""
        The XLMForMultipleChoice forward method, overrides the __call__() special method.
        Args:
            input_ids (Tensor):
                See :class:`XLMModel` and shape as [batch_size, num_choice, sequence_length].
            langs(Tensor, optional):
                See :class:`XLMModel` and shape as [batch_size, num_choice, sequence_length].
            attention_mask (Tensor, optional):
                See :class:`XLMModel` and shape as [batch_size, num_choice, sequence_length].
            position_ids (Tensor, optional):
                See :class:`XLMModel` and shape as [batch_size, num_choice, sequence_length].
            lengths (Tensor, optional):
                See :class:`XLMModel` and shape as [batch_size, num_choice].

        Returns:
            reshaped_logits (Tensor):
                A tensor of the multiple choice classification logits.
                Shape as `[batch_size, num_choice]` and dtype as `float32`.

        Example:
            .. code-block::

                import paddle
                from paddlenlp.transformers import XLMForMultipleChoice, XLMTokenizer
                from paddlenlp.data import Pad

                tokenizer = XLMTokenizer.from_pretrained("xlm-mlm-tlm-xnli15-1024")
                model = XLMForMultipleChoice.from_pretrained("xlm-mlm-tlm-xnli15-1024", num_choices=2)

                data = [
                    {
                        "question": "how do you turn on an ipad screen?",
                        "answer1": "press the volume button.",
                        "answer2": "press the lock button.",
                        "label": 1,
                    },
                    {
                        "question": "how do you indent something?",
                        "answer1": "leave a space before starting the writing",
                        "answer2": "press the spacebar",
                        "label": 0,
                    },
                ]
                text = []
                text_pair = []
                for d in data:
                    text.append(d["question"])
                    text_pair.append(d["answer1"])
                    text.append(d["question"])
                    text_pair.append(d["answer2"])

                inputs = tokenizer(text, text_pair, lang="en")
                input_ids = Pad(axis=0, pad_val=tokenizer.pad_token_id)(inputs["input_ids"])
                input_ids = paddle.to_tensor(input_ids, dtype="int64")
                langs = paddle.ones_like(input_ids) * tokenizer.lang2id["en"]

                reshaped_logits = model(
                    input_ids=input_ids,
                    langs=langs,
                )
        """
        # input_ids: [bs, num_choice, seqlen]
        input_ids = input_ids.reshape(
            shape=(-1, paddle.shape(input_ids)[-1])
        )  # flat_input_ids: [bs*num_choice, seqlen]

        if langs is not None:
            langs = langs.reshape(shape=(-1, paddle.shape(langs)[-1]))

        if attention_mask is not None:
            attention_mask = attention_mask.reshape(shape=(-1, paddle.shape(attention_mask)[-1]))

        if position_ids is not None:
            position_ids = position_ids.reshape(shape=(-1, paddle.shape(position_ids)[-1]))

        if lengths is not None:
            lengths = lengths.reshape(shape=(-1,))

        sequence_output = self.xlm(
            input_ids, langs=langs, attention_mask=attention_mask, position_ids=position_ids, lengths=lengths
        )[0]
        sequence_output = self.dropout(sequence_output)
        pooled_output = sequence_output[:, 0]

        logits = self.classifier(pooled_output)  # logits: [bs*num_choice, 1]
        reshaped_logits = logits.reshape(shape=(-1, self.num_choices))  # logits: [bs, num_choice]

        return reshaped_logits



# Copyright (c) 2021 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import paddle

from . import optimizer
from .distributed import *
from .einsum import *

# isort: split
from .fast_transformer.transformer.decoding import *

# isort: split
from .fast_transformer.transformer.decoder import *
from .fast_transformer.transformer.encoder import *
from .fast_transformer.transformer.fast_transformer import *

paddle.nn.TransformerEncoderLayer._ft_forward = encoder_layer_forward  # noqa F405
paddle.nn.TransformerEncoder._ft_forward = encoder_forward  # noqa F405

paddle.nn.TransformerEncoderLayer._ori_forward = paddle.nn.TransformerEncoderLayer.forward
paddle.nn.TransformerEncoder._ori_forward = paddle.nn.TransformerEncoder.forward



# Copyright (c) 2021 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import math

from paddle.optimizer.lr import LambdaDecay, LRScheduler

__all__ = [
    "LinearDecayWithWarmup",
    "ConstScheduleWithWarmup",
    "CosineDecayWithWarmup",
    "PolyDecayWithWarmup",
    "CosineAnnealingWithWarmupDecay",
    "LinearAnnealingWithWarmupDecay",
]


def is_integer(number):
    if sys.version > "3":
        return isinstance(number, int)
    return isinstance(number, (int, long))


class CosineAnnealingWithWarmupDecay(LRScheduler):
    def __init__(self, max_lr, min_lr, warmup_step, decay_step, last_epoch=-1, verbose=False):
        self.decay_step = decay_step
        self.warmup_step = warmup_step
        self.max_lr = max_lr
        self.min_lr = min_lr
        super(CosineAnnealingWithWarmupDecay, self).__init__(max_lr, last_epoch, verbose)

    def get_lr(self):
        if self.warmup_step > 0 and self.last_epoch <= self.warmup_step:
            return float(self.max_lr) * (self.last_epoch) / self.warmup_step

        if self.last_epoch > self.decay_step:
            return self.min_lr

        num_step_ = self.last_epoch - self.warmup_step
        decay_step_ = self.decay_step - self.warmup_step
        decay_ratio = float(num_step_) / float(decay_step_)
        coeff = 0.5 * (math.cos(math.pi * decay_ratio) + 1.0)
        return self.min_lr + coeff * (self.max_lr - self.min_lr)


class LinearAnnealingWithWarmupDecay(LRScheduler):
    def __init__(self, max_lr, min_lr, warmup_step, decay_step, last_epoch=-1, verbose=False):

        self.decay_step = decay_step
        self.warmup_step = warmup_step
        self.max_lr = max_lr
        self.min_lr = min_lr
        super(LinearAnnealingWithWarmupDecay, self).__init__(max_lr, last_epoch, verbose)

    def get_lr(self):
        if self.warmup_step > 0 and self.last_epoch <= self.warmup_step:
            return float(self.max_lr) * (self.last_epoch) / self.warmup_step

        if self.last_epoch > self.decay_step:
            return self.min_lr

        num_step_ = self.last_epoch - self.warmup_step
        decay_step_ = self.decay_step - self.warmup_step
        decay_ratio = float(num_step_) / float(decay_step_)
        coeff = 1.0 - decay_ratio
        return self.min_lr + coeff * (self.max_lr - self.min_lr)


class LinearDecayWithWarmup(LambdaDecay):
    """
    Creates a learning rate scheduler, which increases learning rate linearly
    from 0 to given `learning_rate`, after this warmup period learning rate
    would be decreased linearly from the base learning rate to 0.

    Args:
        learning_rate (float):
            The base learning rate. It is a python float number.
        total_steps (int):
            The number of training steps.
        warmup (int or float):
            If int, it means the number of steps for warmup. If float, it means
            the proportion of warmup in total training steps.
        last_epoch (int, optional):
            The index of last epoch. It can be set to restart training. If
            None, it means initial learning rate.
            Defaults to -1.
        verbose (bool, optional):
            If True, prints a message to stdout for each update.
            Defaults to False.

    Examples:

        .. code-block:: python

            from paddlenlp.transformers import LinearDecayWithWarmup
            lr, warmup_steps, max_steps = 0.1, 100, 1000
            lr_scheduler = LinearDecayWithWarmup(lr, max_steps, warmup_steps)

    """

    def __init__(self, learning_rate, total_steps, warmup, last_epoch=-1, verbose=False):
        warmup_steps = warmup if is_integer(warmup) else int(math.floor(warmup * total_steps))

        def lr_lambda(current_step):
            if current_step < warmup_steps:
                return float(current_step) / float(max(1, warmup_steps))
            return max(0.0, float(total_steps - current_step) / float(max(1, total_steps - warmup_steps)))

        super(LinearDecayWithWarmup, self).__init__(learning_rate, lr_lambda, last_epoch, verbose)


class ConstScheduleWithWarmup(LambdaDecay):
    """
    Creates a learning rate scheduler, which increases learning rate linearly
    from 0 to given `learning_rate` during warmup periods and keeps learning
    rate a constant after that.

    Args:
        learning_rate (float):
            The base learning rate. It is a python float number.
        warmup (int or float):
            If int, it means the number of steps for warmup. If float, it means
            the proportion of warmup in total training steps.
        total_steps (int, optional):
            The number of training steps. If `warmup` is a float number,
            `total_steps` must be provided.
            Defaults to None.
        last_epoch (int, optional):
            The index of last epoch. It can be set to restart training. If
            None, it means initial learning rate.
            Defaults to -1.

    Examples:

        .. code-block:: python

            from paddlenlp.transformers import ConstScheduleWithWarmup
            lr, warmup_steps = 0.1, 100
            lr_scheduler = ConstScheduleWithWarmup(lr, warmup_steps)

    """

    def __init__(self, learning_rate, warmup, total_steps=None, last_epoch=-1, verbose=False):
        warmup_steps = warmup if is_integer(warmup) else int(math.floor(warmup * total_steps))
        if is_integer(warmup):
            warmup_steps = warmup
        elif total_steps:
            warmup_steps = int(math.floor(warmup * total_steps))
        else:
            raise ValueError(
                "Please provide total steps if `warmup` is a float number , or provide integer for argument `warmup`."
            )

        def lr_lambda(current_step):
            if current_step < warmup_steps:
                return float(current_step) / float(max(1.0, warmup_steps))
            return 1.0

        super(ConstScheduleWithWarmup, self).__init__(learning_rate, lr_lambda, last_epoch, verbose)


class CosineDecayWithWarmup(LambdaDecay):
    """
    Creates a learning rate scheduler, which increases learning rate linearly
    from 0 to given `learning_rate`, after this warmup period learning rate
    would be decreased following the values of the cosine function. If
    `with_hard_restarts` is True, the cosine function could have serveral hard
    restarts.

    Args:
        learning_rate (float):
            The base learning rate. It is a python float number.
        total_steps (int):
            The number of training steps.
        warmup (int or float):
            If int, it means the number of steps for warmup. If float, it means
            the proportion of warmup in total training steps.
        with_hard_restarts (bool):
            Whether cosine function has several hard restarts.
            Defaults to False.
        num_cycles (int or float, optional):
            If `with_hard_restarts` is False, it means the number of waves in
            cosine scheduler and should be an integer number and defaults to 1.
            If `with_hard_restarts` is True, it means the number of hard
            restarts to use and should be a float number and defaults to be 0.5.
            Defaults to None.
        last_epoch (int, optional):
            The index of last epoch. It can be set to restart training. If
            None, it means initial learning rate.
            Defaults to -1.

    Examples:

        .. code-block:: python

            from paddlenlp.transformers import CosineDecayWithWarmup
            lr, warmup_steps, max_steps = 0.1, 100, 1000
            lr_scheduler = CosineDecayWithWarmup(lr, max_steps, warmup_steps)

    """

    def __init__(
        self,
        learning_rate,
        total_steps,
        warmup,
        with_hard_restarts=False,
        num_cycles=None,
        last_epoch=-1,
        verbose=False,
    ):
        warmup_steps = warmup if is_integer(warmup) else int(math.floor(warmup * total_steps))
        # Input check
        if num_cycles is not None:
            assert (
                not with_hard_restarts
                and isinstance(num_cycles, int)
                or with_hard_restarts
                and isinstance(num_cycles, float)
            ), "`num_circles` should be an integer while `with_hard_restarts` is False, an float while `with_hard_restarts` is True."
        else:
            num_cycles = 1 if not with_hard_restarts else 0.5

        def lr_lambda(current_step):
            if current_step < warmup_steps:
                return float(current_step) / float(max(1, warmup_steps))

            progress = float(current_step - warmup_steps) / float(max(1, total_steps - warmup_steps))

            if with_hard_restarts:
                if progress >= 1.0:
                    return 0.0
                return max(0.0, 0.5 * (1.0 + math.cos(math.pi * ((float(num_cycles) * progress) % 1.0))))

            return max(0.0, 0.5 * (1.0 + math.cos(math.pi * float(num_cycles) * 2.0 * progress)))

        super(CosineDecayWithWarmup, self).__init__(learning_rate, lr_lambda, last_epoch, verbose)


class PolyDecayWithWarmup(LambdaDecay):
    """
    Creates a learning rate scheduler, which increases learning rate linearly
    from 0 to given `lr_init`, after this warmup period learning rate would
    be decreased as a polynomial decay from the base learning rate to the end
    learning rate `lr_end`.

    Args:
        learning_rate (float):
            The base learning rate. It is a python float number.
        total_steps (int):
            The number of training steps.
        warmup (int or float):
            If int, it means the number of steps for warmup. If float, it means
            the proportion of warmup in total training steps.
        lr_end (float, optional):
            The end learning rate.
            Defaults to 1e-7.
        power (float, optional):
            Power factor.
            Defaults to 1.0.
        last_epoch (int, optional):
            The index of last epoch. It can be set to restart training. If
            None, it means initial learning rate.
            Defaults to -1.

    Examples:

        .. code-block:: python

            from paddlenlp.transformers import PolyDecayWithWarmup
            lr, lr_end, warmup_steps, max_steps = 0.1, 1e-6, 100, 1000
            lr_scheduler = PolyDecayWithWarmup(lr, max_steps, warmup_steps, lr_end)

    """

    def __init__(self, learning_rate, total_steps, warmup, lr_end=1e-7, power=1.0, last_epoch=-1, verbose=False):
        lr_init = learning_rate
        assert (
            lr_init > lr_end
        ), f"`lr_end` must be be smaller than `learning_rate`. But `lr_end` is {lr_end} while `learning_rate` is {lr_init}."
        warmup_steps = warmup if is_integer(warmup) else int(math.floor(warmup * total_steps))

        def lr_lambda(current_step):
            if current_step < warmup_steps:
                return float(current_step) / float(max(1, warmup_steps))
            elif current_step > total_steps:
                return lr_end / lr_init  # it multiplies by lr_init equals to lr_end
            else:
                lr_range = lr_init - lr_end
                decay_steps = total_steps - warmup_steps
                pct_remaining = 1 - (current_step - warmup_steps) / decay_steps
                decay = lr_range * pct_remaining**power + lr_end
                return decay / lr_init  # it multiplies by lr_init equals to decay

        super(PolyDecayWithWarmup, self).__init__(lr_init, lr_lambda, last_epoch, verbose)



# Copyright (c) 2021 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sentencepiece as spm

from ..albert.tokenizer import AlbertEnglishTokenizer

__all__ = ["ReformerTokenizer"]

PRETRAINED_POSITIONAL_EMBEDDINGS_SIZES = {"reformer-enwik8": 65536, "reformer-crime-and-punishment": 524288}


class ReformerTokenizer(AlbertEnglishTokenizer):
    """
    Constructs a Reformer tokenizer based on SentencePiece .
    This tokenizer inherits from :class:`~paddlenlp.transformers.tokenizer_utils.PretrainedTokenizer`
    which contains most of the main methods. For more information regarding those methods,
    please refer to this superclass.

    Args:
        sentencepiece_model_file (str):
            The vocabulary file (ends with '.spm') required to instantiate
            a `SentencePiece <https://github.com/google/sentencepiece>`__ tokenizer.
        do_lower_case (bool):
            Whether or not to lowercase the input when tokenizing. Defaults to `False`.
        remove_space (bool):
            Whether or note to remove space when tokenizing. Defaults to `True`.
        keep_accents (bool):
            Whether or note to keep accents when tokenizing. Defaults to `False`.
        eos_token (str):
            A special token representing the *eos (end-of-sentence)* token.
            Defaults to "</s>".
        unk_token (str):
            A special token representing the *unknown (out-of-vocabulary)* token.
            An unknown token is set to be `unk_token` inorder to be converted to an ID.
            Defaults to "<unk>".
        pad_token (str):
            A special token used to make arrays of tokens the same size for batching purposes.
            Defaults to "<unk>".

    """

    resource_files_names = {
        "sentencepiece_model_file": "spiece.model",
    }
    pretrained_resource_files_map = {
        "sentencepiece_model_file": {
            "reformer-crime-and-punishment": "http://paddlenlp.bj.bcebos.com/models/transformers/reformer/reformer-crime-and-punishment/spiece.model",
        },
    }

    pretrained_init_configuration = {
        "reformer-crime-and-punishment": {"do_lower_case": False},
    }

    max_model_input_sizes = PRETRAINED_POSITIONAL_EMBEDDINGS_SIZES

    def __init__(
        self,
        sentencepiece_model_file,
        do_lower_case=False,
        remove_space=True,
        keep_accents=False,
        eos_token="</s>",
        unk_token="<unk>",
        pad_token="<unk>",
        **kwargs
    ):

        self.do_lower_case = do_lower_case
        self.remove_space = remove_space
        self.keep_accents = keep_accents
        self.sentencepiece_model_file = sentencepiece_model_file
        self.sp_model = spm.SentencePieceProcessor()
        self.sp_model.Load(sentencepiece_model_file)

    def __call__(
        self,
        text,
        text_pair=None,
        max_seq_len=None,
        stride=0,
        is_split_into_words=False,
        pad_to_max_seq_len=False,
        truncation_strategy="longest_first",
        return_position_ids=False,
        return_token_type_ids=False,
        return_attention_mask=True,
        return_length=False,
        return_overflowing_tokens=False,
        return_special_tokens_mask=False,
    ):
        return super(ReformerTokenizer, self).__call__(
            text,
            text_pair,
            max_seq_len,
            stride,
            is_split_into_words,
            pad_to_max_seq_len,
            truncation_strategy,
            return_position_ids,
            return_token_type_ids,
            return_attention_mask,
            return_length,
            return_overflowing_tokens,
            return_special_tokens_mask,
        )

    def build_inputs_with_special_tokens(self, token_ids_0, token_ids_1=None):
        """
        Build model inputs from a sequence or a pair of sequence.

        An Reformer sequence has the following format:

        - single sequence:      ``X``
        - pair of sequences:        ``A B ``

        Args:
            token_ids_0 (List[int]):
                List of IDs to which the special tokens will be added.
            token_ids_1 (List[int], optional):
                Optional second list of IDs for sequence pairs. Defaults to None.

        Returns:
            List[int]: List of input_id with the appropriate special tokens.

        """
        if token_ids_1 is None:
            return token_ids_0
        return token_ids_0 + token_ids_1

    def create_token_type_ids_from_sequences(self, token_ids_0, token_ids_1=None):
        """
        Create a mask from the two sequences.

        If `token_ids_1` is `None`, this method only returns the first portion of the mask (0s).

        Args:
            token_ids_0 (List[int]):
                List of IDs.
            token_ids_1 (List[int], optional):
                Optional second list of IDs for sequence pairs.

        Returns:
            List[int]: List of token_type_id according to the given sequence(s).

        """
        if token_ids_1 is None:
            return len(token_ids_0) * [0]
        return len(token_ids_0) * [0] + len(token_ids_1) * [1]
